@@ -164,9 +164,17 @@
 #define MUSE_HDR_FLUX_SCALEi       "ESO DRS MUSE FLUX SCALE%d"
 #define MUSE_HDR_FLUX_SCALEi_C     "flux scale factor applied"
 
-/* FITS header keyword to list the filter   *
- * name in the collapsed images (IMAGE_FOV) */
-#define MUSE_HDR_FILTER "ESO DRS MUSE FILTER NAME"
+/* FITS header keyword to list the filter name and *
+ * properties in the collapsed images (IMAGE_FOV)  */
+#define MUSE_HDR_FILTER          "ESO DRS MUSE FILTER NAME"
+#define MUSE_HDR_FILTER_C        "filter name used"
+#define MUSE_HDR_FILTER_ZPVEGA   "ESO DRS MUSE FILTER ZPVEGA"
+#define MUSE_HDR_FILTER_ZPVEGA_C "[mag] Vega-mag zeropoint"
+#define MUSE_HDR_FILTER_ZPAB     "ESO DRS MUSE FILTER ZPAB"
+#define MUSE_HDR_FILTER_ZPAB_C   "[mag] AB-mag zeropoint"
+#define MUSE_HDR_FILTER_FFRAC    "ESO DRS MUSE FILTER FRACTION"
+#define MUSE_HDR_FILTER_FFRAC_C  "[%] fraction of filter covered by data"
+#define MUSE_HDR_FILTER_REGEXP   "^ESO DRS MUSE FILTER "
 
 /* informational keywords used when computing the WCS solution */
 #define MUSE_HDR_WCS_DETSIGMA       "ESO DRS MUSE WCS DETSIGMA"
@@ -321,6 +329,11 @@
 #define QC_ALIGN_DDEC_MEAN         "ESO QC EXPALIGN OFFSET DEC MEAN"
 #define QC_ALIGN_DDEC_STDEV        "ESO QC EXPALIGN OFFSET DEC STDEV"
 
+/* Standard star throughput estimates */
+#define QC_STD_NAME          "ESO QC STANDARD STARNAME"
+#define QC_STD_THRU          "ESO QC STANDARD THRU%04.0f"
+#define QC_STD_ZP            "ESO QC STANDARD ZP %s"
+
 /* Astrometric solution QC headers */
 #define QC_ASTROMETRY_NSTARS "ESO QC ASTRO NSTARS"
 #define QC_ASTROMETRY_SCX    "ESO QC ASTRO SCALE X"
