@@ -2,7 +2,7 @@
 /* vim:set sw=2 sts=2 et cin: */
 /*
  * This file is part of the MUSE Instrument Pipeline
- * Copyright (C) 2005-2014 European Southern Observatory
+ * Copyright (C) 2005-2017 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,6 +30,7 @@
 #include "muse_image.h"
 #include "muse_pixtable.h"
 #include "muse_resampling.h"
+#include "muse_table.h"
 
 /*----------------------------------------------------------------------------*
  *                              Defines                                       *
@@ -42,12 +43,24 @@ static const double kMuseFluxStatFactor = 1e40;
 
 /* FITS headers for storing detected sources found *
  * by the standard star flux integration routines  */
-#define MUSE_HDR_FLUX_NOBJ      "ESO DRS MUSE FLUX NOBJ"
-#define MUSE_HDR_FLUX_OBJn_X    "ESO DRS MUSE FLUX OBJ%d X"
-#define MUSE_HDR_FLUX_OBJn_Y    "ESO DRS MUSE FLUX OBJ%d Y"
-#define MUSE_HDR_FLUX_OBJn_RA   "ESO DRS MUSE FLUX OBJ%d RA"
-#define MUSE_HDR_FLUX_OBJn_DEC  "ESO DRS MUSE FLUX OBJ%d DEC"
-#define MUSE_HDR_FLUX_OBJn_FLUX "ESO DRS MUSE FLUX OBJ%d INTFLUX"
+#define MUSE_HDR_FLUX_NOBJ        "ESO DRS MUSE FLUX NOBJ"
+#define MUSE_HDR_FLUX_NOBJ_C      "Number of objects detected in standard star cube"
+#define MUSE_HDR_FLUX_OBJn_X      "ESO DRS MUSE FLUX OBJ%d X"
+#define MUSE_HDR_FLUX_OBJn_X_C    "[pix] Object x position in cube"
+#define MUSE_HDR_FLUX_OBJn_Y      "ESO DRS MUSE FLUX OBJ%d Y"
+#define MUSE_HDR_FLUX_OBJn_Y_C    "[pix] Object y position in cube"
+#define MUSE_HDR_FLUX_OBJn_RA     "ESO DRS MUSE FLUX OBJ%d RA"
+#define MUSE_HDR_FLUX_OBJn_RA_C   "[deg] Approximate RA coordinate of object"
+#define MUSE_HDR_FLUX_OBJn_DEC    "ESO DRS MUSE FLUX OBJ%d DEC"
+#define MUSE_HDR_FLUX_OBJn_DEC_C  "[deg] Approximate DEC coordinate of object"
+#define MUSE_HDR_FLUX_OBJn_FLUX   "ESO DRS MUSE FLUX OBJ%d INTFLUX"
+#define MUSE_HDR_FLUX_OBJn_FLUX_C "[10**(-20)*erg/s/cm**2] Total flux integrated for this object"
+#define MUSE_HDR_FLUX_NSEL        "ESO DRS MUSE FLUX NSEL"
+#define MUSE_HDR_FLUX_NSEL_C      "Object (%s) selected as standard star"
+
+/* FITS header to mark flat-field spectrum corrected products */
+#define MUSE_HDR_FLUX_FFCORR   "ESO DRS MUSE FLUX FFCORR" /* bool */
+#define MUSE_HDR_FLUX_FFCORR_C "Data was flat-field spectrum corrected"
 
 /*----------------------------------------------------------------------------*
  *                           Data structures                                  *
@@ -68,15 +81,17 @@ typedef struct {
   /** the three extension muse_image to store the *
    *  integrated fluxes and their error estimates */
   muse_image *intimage;
+  /** the flux reference curve of the standard star */
+  cpl_table *reference;
   /** the sensitivity (ratio of expected to measured flux in magnitudes), *
    *  plus additional data in table form                                  */
   cpl_table *sensitivity;
-  /** the flux response curve in table form */
-  cpl_table *response;
+  /** the flux response curve in table form (with header) */
+  muse_table *response;
+  /** the telluric correction in table form (with header) */
+  muse_table *telluric;
   /** the regions of the telluric bands in table form */
   cpl_table *tellbands;
-  /** the telluric correction in table form */
-  cpl_table *telluric;
   /** the right ascension of the reference object in degrees */
   double raref;
   /** the declination of the reference object in degrees */
@@ -150,6 +165,9 @@ cpl_error_code muse_flux_integrate_std(muse_pixtable *, muse_flux_profile_type,
 cpl_error_code muse_flux_response_compute(muse_flux_object *, muse_flux_selection_type, double, const cpl_table *, const cpl_table *, const cpl_table *);
 cpl_error_code muse_flux_get_response_table(muse_flux_object *, muse_flux_smooth_type);
 cpl_error_code muse_flux_get_telluric_table(muse_flux_object *);
-cpl_error_code muse_flux_calibrate(muse_pixtable *, const cpl_table *, const cpl_table *, const cpl_table *);
+cpl_error_code muse_flux_compute_qc(muse_flux_object *);
+cpl_error_code muse_flux_compute_qc_zp(muse_flux_object *, const muse_table *, const char *);
+
+cpl_error_code muse_flux_calibrate(muse_pixtable *, const muse_table *, const cpl_table *, const muse_table *);
 
 #endif /* MUSE_FLUX_H */
