@@ -2,7 +2,7 @@
 /* vim:set sw=2 sts=2 et cin: */
 /*
  * This file is part of the MUSE Instrument Pipeline
- * Copyright (C) 2005-2014 European Southern Observatory
+ * Copyright (C) 2005-2015 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -114,7 +114,8 @@ muse_image_delete(muse_image *aImage)
  */
 /*---------------------------------------------------------------------------*/
 static muse_image *
-muse_image_load_internal(const char *aFilename, unsigned char aIFU, const char *aID)
+muse_image_load_internal(const char *aFilename, unsigned char aIFU,
+                         const char *aID)
 {
   muse_image *image = muse_image_new();
 
@@ -131,6 +132,10 @@ muse_image_load_internal(const char *aFilename, unsigned char aIFU, const char *
   /* now load the image data from the there extensions */
   char extname[KEYWORD_LENGTH];
   if (aIFU) {
+    /* first set the EXTNAME keyword in the output header */
+    snprintf(extname, KEYWORD_LENGTH, "CHAN%02hhu", aIFU);
+    cpl_propertylist_update_string(image->header, "EXTNAME", extname);
+    /* then set up the extension name to search for */
     snprintf(extname, KEYWORD_LENGTH, "CHAN%02hhu.%s", aIFU, EXTNAME_DATA);
   } else {
     snprintf(extname, KEYWORD_LENGTH, "%s", EXTNAME_DATA);
@@ -147,15 +152,15 @@ muse_image_load_internal(const char *aFilename, unsigned char aIFU, const char *
   cpl_propertylist *hdata = cpl_propertylist_load(aFilename, extension);
   if (cpl_propertylist_has(hdata, "BUNIT")) { /* backward compatibility */
     cpl_propertylist_append_string(image->header, "BUNIT",
-                                   cpl_propertylist_get_string(hdata, "BUNIT"));
+                                   muse_pfits_get_bunit(hdata));
     cpl_propertylist_set_comment(image->header, "BUNIT",
                                  cpl_propertylist_get_comment(hdata, "BUNIT"));
   } else {
     cpl_msg_warning(aID, "No BUNIT given in extension %d [%s] of \"%s\"!",
                     extension, extname, aFilename);
   }
-  /* When reading from extensions, we also need to merge in all *
-   * HIERARCH keywords from the data extension. */
+  /* When reading from extensions, we also need to merge in all     *
+   * HIERARCH keywords from the data extension and add the EXTNAME. */
   if (aIFU) {
     cpl_propertylist_erase_regexp(hdata, "^ESO ", 1);
     cpl_propertylist_append(image->header, hdata);
@@ -386,6 +391,7 @@ muse_image_load_from_raw(const char *aFilename, int aExtension)
 
   The data unit ("BUNIT") of the DATA and STAT extensions are set to be the
   incoming BUNIT (from the header component) and (BUNIT)**2, respectively.
+  BUNIT and WCS keys are removed from the primary header before saving.
 
   @error{return CPL_ERROR_NULL_INPUT,
          aImage\, its data component\, or aFilename are NULL}
@@ -406,6 +412,8 @@ muse_image_save(muse_image *aImage, const char *aFilename)
    * so save a header without it                           */
   cpl_propertylist *header = cpl_propertylist_duplicate(aImage->header);
   cpl_propertylist_erase(header, "BUNIT");
+  /* do not save the WCS keywords, either */
+  cpl_propertylist_erase_regexp(header, MUSE_WCS_KEYS, 0);
 
   /* Save the FITS header first, as the primary header of the file. *
    * The extensions will contain only minimal image headers.        */
@@ -427,7 +435,7 @@ muse_image_save(muse_image *aImage, const char *aFilename)
 
   cpl_propertylist_append_string(extheader, "EXTNAME", EXTNAME_DATA);
   cpl_propertylist_set_comment(extheader, "EXTNAME", EXTNAME_DATA_COMMENT);
-  const char *unit = cpl_propertylist_get_string(aImage->header, "BUNIT"),
+  const char *unit = muse_pfits_get_bunit(aImage->header),
              *ucomment = cpl_propertylist_get_comment(aImage->header, "BUNIT");
   cpl_propertylist_append_string(extheader, "BUNIT", unit);
   cpl_propertylist_set_comment(extheader, "BUNIT", ucomment);
@@ -457,7 +465,7 @@ muse_image_save(muse_image *aImage, const char *aFilename)
       cpl_propertylist_delete(extheader);
       return error;
     } /* if error */
-  } /* if stat */
+  } /* if dq */
 
   if (aImage->stat) {
     cpl_propertylist_set_string(extheader, "EXTNAME", EXTNAME_STAT);
@@ -804,8 +812,7 @@ muse_image_adu_to_count(muse_image *aImage)
 {
   cpl_ensure_code(aImage && aImage->header, CPL_ERROR_NULL_INPUT);
   cpl_ensure_code(cpl_propertylist_has(aImage->header, "BUNIT") &&
-                  !strncmp(cpl_propertylist_get_string(aImage->header, "BUNIT"),
-                           "adu", 4),
+                  !strncmp(muse_pfits_get_bunit(aImage->header), "adu", 4),
                   CPL_ERROR_INCOMPATIBLE_INPUT);
   int nx = cpl_image_get_size_x(aImage->data);
   float *data = cpl_image_get_data_float(aImage->data),
