@@ -95,13 +95,26 @@ static const double kMuseSliceSlitWidthA = 2.61;
 /* MUSE nominal spectral range in Angstrom (extended wavelength range, WFM!) */
 static const float kMuseNominalLambdaMin = 4650.;
 static const float kMuseNominalLambdaMax = 9300.;
-/* lower limit of nominal wavelength range with detectable flux */
+/* approximate lower limit of nominal wavelength range with detectable flux */
 static const float kMuseNominalCutoff = 4750;
+static const float kMuseAOCutoff = 4690;
 /* where the blue cutoff filter really kicks in */
 static const float kMuseNominalCutoffKink = 4775.;
+static const float kMuseAOCutoffKink = 4700.;
 /* most extreme wavelength ranges possibly recorded on the MUSE CCDs */
 static const float kMuseLambdaMinX = 4150.;
 static const float kMuseLambdaMaxX = 9800.;
+/* useful wavelength ranges:                         *
+ * common, and illuminated ranges between all slices */
+static const float kMuseUsefulELambdaMin = 4600.;
+static const float kMuseUsefulNLambdaMin = 4750.;
+static const float kMuseUsefulAOLambdaMin = 4700.;
+static const float kMuseUsefulLambdaMax = 9350.;
+/* range where the NaD notch filters absorbs significant flux */
+static const float kMuseNaLambdaMin = 5755.; /* for WFM-AO-E */
+static const float kMuseNaLambdaMax = 6008.;
+static const float kMuseNa2LambdaMin = 5805.; /* for WFM-AO-N */
+static const float kMuseNa2LambdaMax = 5966.;
 /* maximum centering error [pix] to allow for arc lines */
 static const float kMuseArcMaxCenteringError = 1.25;
 /* minimum and maximum FWHM [pix] to allow for arc lines */
@@ -316,17 +329,18 @@ static const double kMuseAmplPhysicalSize = 1.; /* [cm**2] */
   @brief    MUSE instrument observing mode
 
   This enum is ordered so that
-  - the extended mode comes first, all nominal setups later
-  - non-AO modes come first and the non-AO modes later
+  - the non-AO extended mode comes first
+  - non-AO modes come first and the AO modes later
+  - extended mode AO comes between both WFM nominal modes
   - WFM comes before NFM
-  In effect one can use < and > operators to check different parts of the mode.
  */
 /*----------------------------------------------------------------------------*/
 typedef enum {
-  MUSE_MODE_WFM_NONAO_X = 0,
+  MUSE_MODE_WFM_NONAO_E = 0,
   MUSE_MODE_WFM_NONAO_N = 1,
-  MUSE_MODE_WFM_AO_N = 2,
-  MUSE_MODE_NFM_AO_N = 3
+  MUSE_MODE_WFM_AO_E = 2,
+  MUSE_MODE_WFM_AO_N = 3,
+  MUSE_MODE_NFM_AO_N = 4
 } muse_ins_mode;
 
 /*----------------------------------------------------------------------------*
