@@ -275,17 +275,19 @@ muse_lsf_create_arcpixtable(muse_imagelist *aImages,
     cpl_free(l);
     cpl_size i_line;
     cpl_size n_entries = 0;
+    muse_ins_mode insmode = muse_pfits_get_mode(pt->header);
     muse_pixtable **slice_pixtable = muse_pixtable_extracted_get_slices(pixtable);
     int n_slices = muse_pixtable_extracted_get_size(slice_pixtable);
     for (i_line = 0; i_line < cpl_table_get_nrow(aArcLines); i_line++) {
-      if (cpl_table_get_int(aArcLines, "quality", i_line, NULL) < aQuality) {
+      double lambda = cpl_table_get_float(aArcLines, "lambda", i_line, NULL);
+      if (!muse_wave_lines_covered_by_data(lambda, insmode) ||
+          cpl_table_get_int(aArcLines, "quality", i_line, NULL) < aQuality) {
         continue;
       }
       const char *n = muse_wave_lines_get_lampname(aArcLines, i_line);
       char *name = cpl_malloc(strlen(n) + 3);
       sprintf(name, "|%s|", n);
       if (strstr(name, lamp)) {
-        double lambda = cpl_table_get_float(aArcLines, "lambda", i_line, NULL);
         int i_slice;
 #if 0 /* no extra levels of parallelization in basic processing recipes */
         #pragma omp parallel for default(none)           /* as req. by Ralf */ \
