@@ -1356,13 +1356,16 @@ muse_pfits_get_mode(const cpl_propertylist *aHeaders)
   if (!strncmp(value, "NFM", 3)) {
     return MUSE_MODE_NFM_AO_N;
   }
-  if (!strncmp(value, "WFM-AO", 6)) {
+  if (!strncmp(value, "WFM-AO-N", 8)) {
     return MUSE_MODE_WFM_AO_N;
   }
+  if (!strncmp(value, "WFM-AO-E", 8)) {
+    return MUSE_MODE_WFM_AO_E;
+  }
   if (!strncmp(value, "WFM-NOAO-N", 10) || !strncmp(value, "WFM-NONAO-N", 11)) {
     return MUSE_MODE_WFM_NONAO_N;
   }
-  return MUSE_MODE_WFM_NONAO_X;
+  return MUSE_MODE_WFM_NONAO_E;
 }
 
 /*----------------------------------------------------------------------------*/
