@@ -2,7 +2,7 @@
 /* vim:set sw=2 sts=2 et cin: */
 /*
  * This file is part of the MUSE Instrument Pipeline
- * Copyright (C) 2005-2014 European Southern Observatory
+ * Copyright (C) 2005-2016 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -41,21 +41,27 @@
 typedef unsigned int uint32_t;
 #endif
 
-/* Table column names of the pixel table, see muse_pixtable_def for comments */
-#define MUSE_PIXTABLE_DATA    "data"
-#define MUSE_PIXTABLE_DQ      "dq"
-#define MUSE_PIXTABLE_STAT    "stat"
-#define MUSE_PIXTABLE_XPOS    "xpos"
-#define MUSE_PIXTABLE_YPOS    "ypos"
-#define MUSE_PIXTABLE_LAMBDA  "lambda"
-#define MUSE_PIXTABLE_ORIGIN  "origin"
+/** @addtogroup muse_pixtable */
+/**@{*/
+
+/* Table columns of a MUSE pixel table. See muse_pixtable_def for comments. */
+#define MUSE_PIXTABLE_DATA    "data" /**< data column of a MUSE pixel table */
+#define MUSE_PIXTABLE_DQ      "dq" /**< data quality column of a MUSE pixel table */
+#define MUSE_PIXTABLE_STAT    "stat" /**< variance column of a MUSE pixel table */
+#define MUSE_PIXTABLE_XPOS    "xpos" /**< x-position column of a MUSE pixel table */
+#define MUSE_PIXTABLE_YPOS    "ypos" /**< y-position column of a MUSE pixel table */
+#define MUSE_PIXTABLE_LAMBDA  "lambda" /**< wavelength column of a MUSE pixel table */
+#define MUSE_PIXTABLE_ORIGIN  "origin" /**< origin column of a MUSE pixel table */
 /* relative weight of a pixel (used for combining exposures) */
-#define MUSE_PIXTABLE_WEIGHT  "weight"
+#define MUSE_PIXTABLE_WEIGHT  "weight" /**< weight column of a MUSE pixel table (optional) */
+
+/* Table columns of the flat-field spectrum of a MUSE pixel table. */
+#define MUSE_PIXTABLE_FF_EXT   "PIXTABLE_FLAT_FIELD" /**< flat-field extension of a MUSE pixel table (optional) */
+#define MUSE_PIXTABLE_FFLAMBDA MUSE_PIXTABLE_LAMBDA /**< wavelength column for the flat-field extension of a MUSE pixel table */
+#define MUSE_PIXTABLE_FFDATA   MUSE_PIXTABLE_DATA/**< wavelength column for the flat-field extension of a MUSE pixel table */
 
 extern const muse_cpltable_def muse_pixtable_def[];
 
-/** @addtogroup muse_pixtable */
-/**@{*/
 /**
   @brief   FITS header keyword for the horizontal slice offset on the CCD.
 
@@ -168,6 +174,9 @@ extern const muse_cpltable_def muse_pixtable_def[];
 #define MUSE_HDR_PT_PREDAR_YHI "ESO DRS MUSE PIXTABLE PREDAR LIMITS YHI"
 
 /* headers and comments to describe the status of the pixel table */
+/** Flat-field corrected pixel table, number is the IFUs that had flat-fields */
+#define MUSE_HDR_PT_FFCORR             "ESO DRS MUSE PIXTABLE FFCORR" /* int */
+#define MUSE_HDR_PT_FFCORR_COMMENT     "Pixel table corrected for flat-field spectrum"
 /** Merged IFUs that went into this pixel table */
 #define MUSE_HDR_PT_MERGED             "ESO DRS MUSE PIXTABLE MERGED" /* int */
 #define MUSE_HDR_PT_MERGED_COMMENT     "Merged IFUs that went into this pixel table"
@@ -240,6 +249,15 @@ typedef struct {
     output file.
    */
   cpl_propertylist *header;
+
+  /**
+    @brief A flat-field spectrum.
+
+    This may contain the (average) flat-field spectrum of an IFU or an
+    exposure, in the format of a two-column table. The columns then are
+    MUSE_PIXTABLE_FFLAMBDA and MUSE_PIXTABLE_FFDATA.
+   */
+  cpl_table *ffspec;
 } muse_pixtable;
 
 /*----------------------------------------------------------------------------*/
@@ -256,6 +274,17 @@ enum muse_pixtable_type {
   MUSE_PIXTABLE_TYPE_FULL
 };
 
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    Type of operation to apply to a MUSE pixel table
+ */
+/*----------------------------------------------------------------------------*/
+typedef enum {
+  MUSE_PIXTABLE_OPERATION_SUBTRACT = 0,
+  MUSE_PIXTABLE_OPERATION_MULTIPLY,
+  MUSE_PIXTABLE_OPERATION_DIVIDE
+} muse_pixtable_operation;
+
 /*----------------------------------------------------------------------------*/
 /**
   @brief    State of the astrometric calibration of a MUSE pixel table
@@ -292,6 +321,7 @@ unsigned int muse_pixtable_get_expnum(muse_pixtable *, cpl_size);
 muse_pixtable *muse_pixtable_create(muse_image *, cpl_table *, cpl_table *, cpl_table *);
 muse_pixtable *muse_pixtable_duplicate(muse_pixtable *);
 void muse_pixtable_delete(muse_pixtable *);
+cpl_error_code muse_pixtable_append_ff(muse_pixtable *, muse_image *, cpl_table *, cpl_table *, float);
 cpl_error_code muse_pixtable_save(muse_pixtable *, const char *);
 muse_pixtable *muse_pixtable_load_window(const char *, cpl_size, cpl_size);
 muse_pixtable *muse_pixtable_load(const char *);
@@ -303,6 +333,7 @@ int muse_pixtable_get_type(muse_pixtable *);
 cpl_size muse_pixtable_get_nrow(const muse_pixtable *);
 cpl_error_code muse_pixtable_compute_limits(muse_pixtable *);
 cpl_error_code muse_pixtable_flux_multiply(muse_pixtable *, double);
+cpl_error_code muse_pixtable_spectrum_apply(muse_pixtable *, const cpl_array *, const cpl_array *, muse_pixtable_operation);
 
 cpl_error_code muse_pixtable_restrict_wavelength(muse_pixtable *, double, double);
 cpl_error_code muse_pixtable_restrict_xpos(muse_pixtable *, double, double);
