@@ -1209,7 +1209,7 @@ muse_postproc_qc_fwhm(muse_processing *aProcessing, muse_datacube *aCube)
     cpl_errorstate_set(es); /* swallow possible errors from creating the WCS */
   }
   if (!wcs || !strncasecmp(muse_pfits_get_ctype(aCube->header, 1), "PIXEL", 5)) {
-    if (muse_pfits_get_mode(aCube->header) > MUSE_MODE_WFM_AO_N) { /* NFM scaling */
+    if (muse_pfits_get_mode(aCube->header) == MUSE_MODE_NFM_AO_N) { /* NFM scaling */
       cd11 = kMuseSpaxelSizeX_NFM;
       cd22 = kMuseSpaxelSizeY_NFM;
     }
@@ -1286,7 +1286,7 @@ muse_postproc_qc_fwhm(muse_processing *aProcessing, muse_datacube *aCube)
   cpl_errorstate state = cpl_errorstate_get();
   cpl_stats *s = cpl_stats_new_from_image(statimage,
                                           CPL_STATS_MEDIAN | CPL_STATS_MAD);
-  cpl_boolean error = cpl_errorstate_is_equal(state);
+  cpl_boolean error = !cpl_errorstate_is_equal(state);
   cpl_errorstate_set(state); /* swallow the error again */
   snprintf(keyword, KEYWORD_LENGTH, QC_POST_FWHM_MEDIAN, prefix);
   if (error || ndet < 3) {
