@@ -437,7 +437,7 @@ muse_processing_recipeinfo(cpl_plugin *aPlugin)
                         "MUSE_PIXTABLE_SAVE_AS_TABLE", "MUSE_COLLAPSE_PIXTABLE",
                         "MUSE_COLLAPSE_USE_VARIANCE",
                         "MUSE_CPL_ERRORSTATE_NDUMP", "MUSE_EXPERT_USER",
-                        "MUSE_BADPIX_FROM_MASKCUBE", "MUSE_DEBUG_QUADRANTS",
+                        "MUSE_DEBUG_QUADRANTS",
                         "MUSE_DEBUG_TRACE", "MUSE_DEBUG_WAVECAL",
                         "MUSE_DEBUG_LSF_FIT", "MUSE_DEBUG_SKY",
                         "MUSE_PLOT_TRACE", "MUSE_DEBUG_DCR", "MUSE_DEBUG_FLUX",
