@@ -146,6 +146,7 @@ muse_sky_lines_save(muse_processing *aProcessing, const cpl_table *aLines,
   rc = muse_cpltable_append_file(aLines, filename,
                                 "LINES", muse_sky_lines_lines_def);
   if (rc == CPL_ERROR_NONE) {
+#pragma omp critical(muse_processing_output_frames)
     cpl_frameset_insert(aProcessing->outframes, frame);
   } else {
     cpl_frame_delete(frame);
@@ -187,23 +188,29 @@ muse_sky_lines_load(muse_processing *aProcessing)
     return NULL;
   }
 
+  // Output message, no matter if unit conversions happen
+  // (so that the following messages can be related to a file name)
+  cpl_msg_info(__func__, "Loaded sky lines from \"%s\"", fn);
   // Convert tables with unscaled flux into ones with scaled flux.
-  if (lines != NULL) {
-    if (strcmp(cpl_table_get_column_unit(lines, "flux"),
-               "erg/(s cm^2 arcsec^2)") == 0) {
-      cpl_msg_info(__func__, "Scaling flux by 1e20");
+  if (lines) {
+    cpl_msg_indent_more();
+    const char *unit = cpl_table_get_column_unit(lines, "flux");
+    if (!unit) {
+      cpl_msg_warning(__func__, "No flux unit given!");
+    }
+    if (unit && strcmp(unit, "erg/(s cm^2 arcsec^2)") == 0) {
+      cpl_msg_info(__func__, "Scaling flux by 1e20.");
       cpl_table_multiply_scalar(lines, "flux", 1e20);
-      cpl_table_set_column_unit(lines, "flux", 
+      cpl_table_set_column_unit(lines, "flux",
                                 "10**(-20)*erg/(s cm^2 arcsec^2)");
     }
-    if (strcmp(cpl_table_get_column_unit(lines, "flux"),
-               "10**(-20)*erg/(s cm^2 arcsec^2)") != 0) {
-      cpl_msg_warning(__func__, "Unsupported flux unit %s",
+    if (unit && strcmp(unit, "10**(-20)*erg/(s cm^2 arcsec^2)") != 0) {
+      cpl_msg_warning(__func__, "Unsupported flux unit \"%s\".",
                       cpl_table_get_column_unit(lines, "flux"));
     }
-  }
+    cpl_msg_indent_less();
+  } /* if lines */
 
-  cpl_msg_info(__func__, "Loaded sky lines from \"%s\"", fn);
   muse_processing_append_used(aProcessing, frame, CPL_FRAME_GROUP_CALIB, 1);
   cpl_frameset_delete(frames);
 
