@@ -541,8 +541,10 @@ muse_sky_subtract_lines_old(muse_pixtable *aPixtable, cpl_table *aLines,
   cpl_size i_slice;
   cpl_msg_info(__func__, "Starting sky subtraction of %"CPL_SIZE_FORMAT" slices",
                n_slices);
+  cpl_boolean debug = getenv("MUSE_DEBUG_SKY")
+                    && atoi(getenv("MUSE_DEBUG_SKY")) > 0;
   #pragma omp parallel for default(none)                 /* as req. by Ralf */ \
-              shared(aLsfParams, aLines, n_slices, slice_pixtable)
+              shared(aLsfParams, aLines, debug, n_slices, slice_pixtable)
   for (i_slice = 0; i_slice < n_slices; i_slice++) {
     uint32_t origin
       = (uint32_t)cpl_table_get_int(slice_pixtable[i_slice]->table,
@@ -557,8 +559,10 @@ muse_sky_subtract_lines_old(muse_pixtable *aPixtable, cpl_table *aLines,
     }
 
     cpl_size nrows = muse_pixtable_get_nrow(slice_pixtable[i_slice]);
-    cpl_msg_debug(__func__, "Sky subtraction of %li pixels for slice #%i.%i",
-                  (long)nrows, ifu, slice);
+    if (debug) {
+      cpl_msg_debug(__func__, "Sky subtraction of %li pixels for slice #%i.%i",
+                    (long)nrows, ifu, slice);
+    }
     cpl_errorstate prestate = cpl_errorstate_get();
     muse_pixtable *slice_pt = slice_pixtable[i_slice];
     cpl_propertylist *order = cpl_propertylist_new();
