@@ -2,7 +2,7 @@
 /* vim:set sw=2 sts=2 et cin: */
 /*
  * This file is part of the MUSE Instrument Pipeline
- * Copyright (C) 2005-2014 European Southern Observatory
+ * Copyright (C) 2005-2017 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -278,7 +278,7 @@ muse_trace_locate_slices(cpl_vector *aRowVec, const unsigned short aNSlices,
 
     /* possibly plot the result */
     char *doplot = getenv("MUSE_PLOT_TRACE");
-    if (doplot && atoi(doplot) & 0x1) {
+    if (doplot && (atoi(doplot) & 0x1)) {
       muse_trace_plot_located_slices(aRowVec, centers, median, mdev, detlimit,
                                      aIFU);
     }
@@ -405,69 +405,100 @@ muse_trace_edgefinder(const cpl_vector *aDataVec, double aFrac,
   /* buffer access to vector */
   const double *ydata = cpl_vector_get_data_const(aDataVec);
 
-  /* start at the center and search outwards for the right-hand edge */
-  int i;
-  for (i = size/2; i < size; i++) {
-    if (ydata[i] < detlimit) {
-      /* i is the element where we fell below the limit. Use this to   *
-       * determine the fractional position where the limit was reached *
-       * using linear interpolation                                    */
-      *aRight = i-1 + (detlimit - ydata[i-1]) / (ydata[i] - ydata[i-1]);
+  /* run the followingn at least once, iterate if the width is too small */
+  int offset = 0; /* do the first iteration starting at the center *
+                   * of the extracted buffer                       */
+  do {
 #if 0
-      cpl_msg_debug(__func__, "r: %d..._%d_, %f/_%f_ ===> %f", i-1, i,
-                    ydata[i-1], ydata[i], *aRight);
+    if (offset > 0) {
+      cpl_msg_debug(__func__, "Iterating edge search with offset = %d pix (IFU "
+                    "%hhu)", offset, aIFU);
+    }
 #endif
-      /* if the found interpolation point is more than 1 pix away *
-       * from the current pixel then the interpolation was wrong! */
-      if (fabs(*aRight - i) > 1.) {
-        /* Only display message and return the fault, if we actually got a few *
-         * pixels away from the position where we started. Otherwise it will   *
-         * just be a dark pixel near the slice center, that we can ignore.     *
-         * This case would be a real problem and should actually not happen!   */
-        if (significant && i - size/2 > 2) {
-          cpl_msg_debug(__func__, "Faulty interpolation of right-hand edge in "
-                        "IFU %hhu: i=%d (start %d), *aRight=%f (%f..%f > %f > "
-                        "%f)", aIFU, i, size/2, *aRight, ydata[i-2], ydata[i-1],
-                        detlimit, ydata[i]);
-          return -11;
-        } /* if real problem */
-        continue;
-      } /* if large offset */
-      break;
-    } /* if data below limit */
-  } /* for i (right-hand part of slice) */
-  /* we arrived at upper limit of vector, so we didn't find the right edge! */
-  if (i == size) {
-    return -1;
-  }
 
-  /* start again at the center and search outwards for the left-hand edge */
-  for (i = size/2; i >= 0; i--) {
-    if (ydata[i] < detlimit) {
-      /* again, use linear interpolation to find the exact position */
-      *aLeft = i + (detlimit - ydata[i]) / (ydata[i+1] - ydata[i]);
+    /* start at the center and search outwards for the right-hand edge */
+    int i;
+    for (i = size/2 + offset; i < size; i++) {
+      if (ydata[i] < detlimit) {
+        /* i is the element where we fell below the limit. Use this to   *
+         * determine the fractional position where the limit was reached *
+         * using linear interpolation                                    */
+        *aRight = i-1 + (detlimit - ydata[i-1]) / (ydata[i] - ydata[i-1]);
 #if 0
-      cpl_msg_debug(__func__, "l: %d..._%d_, %f/_%f_ ===> %f", i+1, i,
-                    ydata[i+1], ydata[i], *aLeft);
+        cpl_msg_debug(__func__, "r: %d..._%d_, %f/_%f_ ===> %f", i-1, i,
+                      ydata[i-1], ydata[i], *aRight);
 #endif
-      if (fabs(*aLeft - i) > 1.) {
-        if (significant && size/2 - i > 2) {
-          /* this would be a real problem, should never happen! */
-          cpl_msg_debug(__func__, "Faulty interpolation of left-hand edge in "
-                        "IFU %hhu: i=%d (start %d), *aLeft=%f (%f < %f < %f..%f"
-                        ")", aIFU, i, size/2, *aLeft, ydata[i], detlimit,
-                        ydata[i+1], ydata[i+2]);
-          return -12;
-        } /* if real problem */
-        continue;
-      } /* if large offset */
+        /* if the found interpolation point is more than 1 pix away *
+         * from the current pixel then the interpolation was wrong! */
+        if (fabs(*aRight - i) > 1.) {
+          /* Only display message and return the fault, if we actually got a few *
+           * pixels away from the position where we started. Otherwise it will   *
+           * just be a dark pixel near the slice center, that we can ignore.     *
+           * This case would be a real problem and should actually not happen!   */
+          if (significant && (i - (size/2 + offset)) > 2) {
+            cpl_msg_debug(__func__, "Faulty interpolation of right-hand edge in "
+                          "IFU %hhu: i=%d (start %d), *aRight=%f (%f..%f > %f > "
+                          "%f)", aIFU, i, size/2 + offset, *aRight, ydata[i-2],
+                          ydata[i-1], detlimit, ydata[i]);
+            return -11;
+          } /* if real problem */
+          continue;
+        } /* if large offset */
+        break;
+      } /* if data below limit */
+    } /* for i (right-hand part of slice) */
+    /* we arrived at upper limit of vector, so we didn't find the right edge! */
+    if (i == size) {
+      return -1;
+    }
+
+    /* start again at the center and search outwards for the left-hand edge */
+    for (i = size/2 - offset; i >= 0; i--) {
+      if (ydata[i] < detlimit) {
+        /* again, use linear interpolation to find the exact position */
+        *aLeft = i + (detlimit - ydata[i]) / (ydata[i+1] - ydata[i]);
+#if 0
+        cpl_msg_debug(__func__, "l: %d..._%d_, %f/_%f_ ===> %f", i+1, i,
+                      ydata[i+1], ydata[i], *aLeft);
+#endif
+        if (fabs(*aLeft - i) > 1.) {
+          if (significant && (size/2 - offset - i) > 2) {
+            /* this would be a real problem, should never happen! */
+            cpl_msg_debug(__func__, "Faulty interpolation of left-hand edge in "
+                          "IFU %hhu: i=%d (start %d), *aLeft=%f (%f < %f < %f..%f"
+                          ")", aIFU, i, size/2 - offset, *aLeft, ydata[i],
+                          detlimit, ydata[i+1], ydata[i+2]);
+            return -12;
+          } /* if real problem */
+          continue;
+        } /* if large offset */
+        break;
+      } /* if data below limit */
+    } /* for i (left-hand part of slice) */
+    /* we arrived at lower limit of vector, so we didn't find the left edge! */
+    if (i == -1) {
+      return -2;
+    }
+
+    /* If we need to iterate, do so with an offset from the center that is 1  *
+     * pixel larger than the smaller of the two distances that we found last. */
+    int offsetold = offset;
+    if ((size/2 - *aLeft) > (*aRight - size/2)) {
+      offset = *aRight - size/2 + 2;
+    } else {
+      offset = size/2 - *aLeft + 2;
+    }
+    if (offset <= offsetold) {
+      /* Ensure that the next iteration starts at least one    *
+       * pixel further, otherwise we might not stop iterating! */
+      offset++;
+    }
+    if (offset > size/2) {
+      /* if the offset is already half the extent of the buffer, *
+       * we won't find the edge(s) any more...                   */
       break;
-    } /* if data below limit */
-  } /* for i (left-hand part of slice) */
-  /* we arrived at lower limit of vector, so we didn't find the left edge! */
-  if (i == -1) {
-    return -2;
-  }
+    }
+  } while ((*aRight - *aLeft + 1) < kMuseSliceLoLikelyWidth || (offset > size/2));
 
 #if 0
   cpl_msg_debug(__func__, "result in IFU %hhu: %f %f --> %f", aIFU, *aLeft,
@@ -637,7 +668,7 @@ muse_trace_refine_edge(cpl_vector *aDiffVec, double *aLeft, double *aRight,
   if (fabs(mid - midpoint) > TRACE_REFINE_MAX_SHIFT) {
     cpl_msg_debug(__func__, "large refined shift around y=%.1f in slice %hu of "
                   "IFU %hhu: %f %f %f (%f %f %f) trace point will not be used",
-                  aY, aIFU, aSlice, left + aOffset, midpoint + aOffset,
+                  aY, aSlice, aIFU, left + aOffset, midpoint + aOffset,
                   right + aOffset, left - *aLeft, mid - midpoint, right - *aRight);
     midpoint = -1.; /* make it fail */
   } else { /* success, add the offset */
@@ -956,13 +987,17 @@ muse_trace(const muse_image *aImage, int aNSum, double aEdgeFrac, int aFitorder,
   cpl_msg_info(__func__, "Working with %hu slices, %d image rows, and %d "
                "tracepoints in IFU %hhu", nsearchslices, ny, npoints, ifu);
 
-  cpl_boolean chan24in2014 = (ifu == 24)
-                           && (!strncmp(muse_pfits_get_dateobs(aImage->header), "2014-", 5) ||
-                               !strncmp(muse_pfits_get_dateobs(aImage->header), "2015-", 5));
-  if (chan24in2014) {
-    cpl_msg_info(__func__, "Using overrides for IFU 24 in 2014/5: due to field "
-                 "vignetting, left-hand edges of slices 37 to 48 may be only "
-                 "approximate!");
+  /* check if this is IFU 24 and data taken since it was installed at Paranal *
+   * in January 2014, because that has the strange vignetting in channel 24   */
+  cpl_boolean chan24paranal = (ifu == 24);
+  const char *dateobs = muse_pfits_get_dateobs(aImage->header);
+  if (chan24paranal && dateobs) { /* it is IFU 24 and we do have a DATE-OBS */
+    chan24paranal = atoi(dateobs) >= 2014;
+  }
+  if (chan24paranal) {
+    cpl_msg_info(__func__, "Using overrides for IFU 24 since 2014: due to field"
+                 " vignetting, left-hand edges (as seen on the CCD) of slices "
+                 "37 to 48 may be only approximate!");
   }
 
   /* duplicate input image to be able to interpolate bad pixels */
@@ -1064,8 +1099,8 @@ muse_trace(const muse_image *aImage, int aNSum, double aEdgeFrac, int aFitorder,
           jlo = j,
           jhi = j + aNSum - 1;
 #if 0
-      cpl_msg_debug(__func__, "slice=%hu, center=%f, cut region: %d,%d,%d,%d",
-                    islice + 1, cpl_vector_get(centers, islice), ilo, ihi, jlo, jhi);
+      cpl_msg_debug(__func__, "slice=%d, center=%f, cut region: %d,%d,%d,%d",
+                    (int)islice + 1, cpl_vector_get(centers, islice), ilo, ihi, jlo, jhi);
 #endif
       cpl_image *tmp = cpl_image_collapse_window_create(image->data,
                                                         ilo, jlo, ihi, jhi,
@@ -1097,7 +1132,7 @@ muse_trace(const muse_image *aImage, int aNSum, double aEdgeFrac, int aFitorder,
       cpl_vector_delete(cut);
 
       /* slices 37 to 48 on the CCD have rounded left edge for IFU 24 */
-      if (midpoint < 0 && midpoint > -2 && islice+1 >= 37 && chan24in2014) {
+      if (midpoint < 0 && midpoint > -2 && islice+1 >= 37 && chan24paranal) {
         cpl_msg_debug(__func__, "IFU24 problem? slice %d, y = %f: refined "
                       "%f < %f < %f", islice+1, ((double)jlo + jhi) / 2.,
                       left, midpoint, right);
@@ -1142,13 +1177,13 @@ muse_trace(const muse_image *aImage, int aNSum, double aEdgeFrac, int aFitorder,
         nfailed++;
       }
 #if 0
-      cpl_msg_debug(__func__, "slice=%hu, nfailed=%d, tracepoint=%d, midpoint="
-                    "%f, y=%d", islice + 1, nfailed, knum, midpoint, j);
+      cpl_msg_debug(__func__, "slice=%d, nfailed=%d, tracepoint=%d, midpoint="
+                    "%f, y=%d", (int)islice + 1, nfailed, knum, midpoint, j);
 #endif
       if (nfailed > 0.1*npoints && midpoint == -1.) {
-        cpl_msg_warning(__func__, "failure %d in slice %hu of IFU %hhu: lost "
+        cpl_msg_warning(__func__, "failure %d in slice %d of IFU %hhu: lost "
                         "trace at y=%d (tracepoint %d of %d)", nfailed,
-                        islice + 1, ifu, j, knum, npoints);
+                        (int)islice + 1, ifu, j, knum, npoints);
       }
 
       /* resize vector and matrix, so that they don't include the failure */
@@ -1168,8 +1203,8 @@ muse_trace(const muse_image *aImage, int aNSum, double aEdgeFrac, int aFitorder,
 
     /* iterate through input points and the fit with XXX 5 sigma rejections */
     const float kWSigma = 5, kRSigma = 5;
-    cpl_msg_debug(__func__, "Working on slice %hu of IFU %hhu (kWSigma=%f, "
-                  "kRSigma=%f)", islice + 1, ifu, kWSigma, kRSigma);
+    cpl_msg_debug(__func__, "Working on slice %d of IFU %hhu (kWSigma=%f, "
+                  "kRSigma=%f)", (int)islice + 1, ifu, kWSigma, kRSigma);
     cpl_vector *mse = cpl_vector_new(MUSE_TRACE_NPOLY);
     cpl_vector_fill(mse, -1.);
     cpl_polynomial **tracefits = muse_trace_iterate_fit(xtrace, ytrace, widths,
@@ -1185,8 +1220,8 @@ muse_trace(const muse_image *aImage, int aNSum, double aEdgeFrac, int aFitorder,
 
     if (!tracefits) {
       /* we need to print an error message and skip the table entries */
-      cpl_msg_error(__func__, "The trace fit in slice %hu of IFU %hhu failed",
-                    islice + 1, ifu);
+      cpl_msg_error(__func__, "The trace fit in slice %d of IFU %hhu failed",
+                    (int)islice + 1, ifu);
       cpl_vector_delete(mse);
       continue;
     }
@@ -1197,8 +1232,8 @@ muse_trace(const muse_image *aImage, int aNSum, double aEdgeFrac, int aFitorder,
     cpl_table_set_float(tracetable, MUSE_TRACE_TABLE_COL_WIDTH, islice, wmean);
     for (ipoly = 0; ipoly < MUSE_TRACE_NPOLY; ipoly++) {
       if (!tracefits[ipoly]) {
-        cpl_msg_error(__func__, "The fit %d in slice %hu of IFU %hhu failed",
-                      ipoly, islice + 1, ifu);
+        cpl_msg_error(__func__, "The fit %d in slice %d of IFU %hhu failed",
+                      ipoly, (int)islice + 1, ifu);
         continue;
       }
       char *colname = cpl_sprintf(MUSE_TRACE_TABLE_COL_MSE, ipoly);
@@ -1215,9 +1250,9 @@ muse_trace(const muse_image *aImage, int aNSum, double aEdgeFrac, int aFitorder,
 #define SLOPE_WARN_LIMIT 0.015
         if (j == 1 && fabs(coeff) > SLOPE_WARN_LIMIT) {
           cpl_msg_warning(__func__, "1st order coefficient of the %s tracing "
-                          "polynomial is unexpectedly large in slice %hu of IFU"
+                          "polynomial is unexpectedly large in slice %d of IFU"
                           " %hhu: |%f| > %f", muse_trace_poly_strings[ipoly],
-                          islice + 1, ifu, coeff, SLOPE_WARN_LIMIT);
+                          (int)islice + 1, ifu, coeff, SLOPE_WARN_LIMIT);
         }
         cpl_table_set_double(tracetable, colname, islice, coeff);
         if (!cpl_errorstate_is_equal(prestate)) {
@@ -1248,8 +1283,8 @@ muse_trace(const muse_image *aImage, int aNSum, double aEdgeFrac, int aFitorder,
     for (islice = 0; islice < cpl_table_get_nrow(tracetable); islice++) {
       double width = cpl_table_get(tracetable, MUSE_TRACE_TABLE_COL_WIDTH, islice, NULL);
       if (width < 40.) {
-        cpl_error_set_message(__func__, CPL_ERROR_ILLEGAL_OUTPUT, "slice %hu of "
-                              "IFU %hhu was narrow (%f), erased it", islice + 1,
+        cpl_error_set_message(__func__, CPL_ERROR_ILLEGAL_OUTPUT, "slice %d of "
+                              "IFU %hhu was narrow (%f), erased it", (int)islice + 1,
                               ifu, width);
         cpl_table_erase_window(tracetable, islice--, 1);
       } /* if */
@@ -1281,8 +1316,8 @@ muse_trace(const muse_image *aImage, int aNSum, double aEdgeFrac, int aFitorder,
         cen = cpl_table_get(row, "tc0_00", 0, NULL); /* new center */
         cpl_table_delete(row);
         cpl_error_set_message(__func__, CPL_ERROR_ILLEGAL_OUTPUT, "slice was "
-                              "missing before slice %hu of IFU %hhu, copied "
-                              "from slice %hu", islice + 1, ifu, iref + 1);
+                              "missing before slice %d of IFU %hhu, copied "
+                              "from slice %d", (int)islice + 1, ifu, (int)iref + 1);
       }
       last = cen;
     } /* for islice */
@@ -1292,8 +1327,8 @@ muse_trace(const muse_image *aImage, int aNSum, double aEdgeFrac, int aFitorder,
                                                  islice, NULL);
       if (sliceno != islice + 1) {
         cpl_msg_warning(__func__, "Resetting entry at table row index %hu to "
-                        "correct slice number in IFU %hhu (%hu instead of %hu)",
-                        islice, ifu, islice + 1, sliceno);
+                        "correct slice number in IFU %hhu (%d instead of %hu)",
+                        islice, ifu, (int)islice + 1, sliceno);
         cpl_table_set_int(tracetable, MUSE_TRACE_TABLE_COL_SLICE_NO, islice,
                           islice + 1);
       }
@@ -1492,6 +1527,7 @@ muse_trace_plot_located_slices(cpl_vector *aRowVec, cpl_vector *aCenters,
   @param   aSlice1    first slice to plot (1 or lower starts with the first one)
   @param   aSlice2    last slice to plot (kMuseSlicesPerCCD or higher ends with
                       the last one)
+  @param   aIFU       the IFU number (only used if > 0)
   @param   aImage     the image to use as background for the plot
   @return  CPL_ERROR_NONE on success a CPL error code on failure
 
@@ -1509,7 +1545,7 @@ muse_trace_plot_located_slices(cpl_vector *aRowVec, cpl_vector *aCenters,
 cpl_error_code
 muse_trace_plot_samples(cpl_table *aSamples, cpl_table *aTrace,
                         unsigned short aSlice1, unsigned short aSlice2,
-                        muse_image *aImage)
+                        unsigned char aIFU, muse_image *aImage)
 {
 #if HAVE_POPEN && HAVE_PCLOSE
   cpl_ensure_code(aSamples, CPL_ERROR_NULL_INPUT);
@@ -1524,10 +1560,14 @@ muse_trace_plot_samples(cpl_table *aSamples, cpl_table *aTrace,
     aSlice2 = kMuseSlicesPerCCD / 2 + 1;
   }
   if (aSlice2 - aSlice1 > 10) {
-    fprintf(stderr, "Warning: plotting %hu slices may take a long time and "
-            "RAM/disk space!\n", aSlice2 - aSlice1 + 1);
+    fprintf(stderr, "Warning: plotting %d slices may take a long time and "
+            "RAM/disk space!\n", (int)aSlice2 - (int)aSlice1 + 1);
   }
-  printf("Plotting slices %hu to %hu\n", aSlice1, aSlice2);
+  printf("Plotting ");;
+  if (aIFU > 0) {
+    printf("IFU %hhu, ", aIFU);
+  }
+  printf("slices %hu to %hu\n", aSlice1, aSlice2);
 
   FILE *gp = popen("gnuplot", "w");
   if (!gp) {
@@ -1640,7 +1680,11 @@ muse_trace_plot_samples(cpl_table *aSamples, cpl_table *aTrace,
   fclose(sf); /* close the file now to unlock it for plotting */
 
   /* plot title */
-  fprintf(gp, "set title \"trace result, slices %hu to %hu\"\n", aSlice1, aSlice2);
+  fprintf(gp, "set title \"trace result, ");
+  if (aIFU > 0) {
+    fprintf(gp, "IFU %hhu, ", aIFU);
+  }
+  fprintf(gp, "slices %hu to %hu\"\n", aSlice1, aSlice2);
   fprintf(gp, "set palette gray\n"); /* alternative palette in greyscale */
   fprintf(gp, "unset key\n");
 
@@ -1701,6 +1745,7 @@ muse_trace_plot_samples(cpl_table *aSamples, cpl_table *aTrace,
   @param   aSlice1    first slice to plot (1 or lower starts with the first one)
   @param   aSlice2    last slice to plot (kMuseSlicesPerCCD or higher ends with
                       the last one)
+  @param   aIFU       the IFU number (only used if > 0)
   @return  CPL_ERROR_NONE on success a CPL error code on failure
 
   Uses color-coded lines/points for each relevant slice, ranging from red on
@@ -1719,7 +1764,7 @@ muse_trace_plot_samples(cpl_table *aSamples, cpl_table *aTrace,
 /*---------------------------------------------------------------------------*/
 cpl_error_code
 muse_trace_plot_widths(cpl_table *aSamples, unsigned short aSlice1,
-                       unsigned short aSlice2)
+                       unsigned short aSlice2, unsigned char aIFU)
 {
 #if HAVE_POPEN && HAVE_PCLOSE
   cpl_ensure_code(aSamples, CPL_ERROR_NULL_INPUT);
@@ -1733,7 +1778,11 @@ muse_trace_plot_widths(cpl_table *aSamples, unsigned short aSlice1,
     aSlice1 = kMuseSlicesPerCCD / 2;
     aSlice2 = kMuseSlicesPerCCD / 2 + 1;
   }
-  printf("Plotting slices %hu to %hu\n", aSlice1, aSlice2);
+  printf("Plotting ");;
+  if (aIFU > 0) {
+    printf("IFU %hhu, ", aIFU);
+  }
+  printf("slices %hu to %hu\n", aSlice1, aSlice2);
 
   FILE *gp = popen("gnuplot", "w");
   if (!gp) {
@@ -1747,8 +1796,11 @@ muse_trace_plot_widths(cpl_table *aSamples, unsigned short aSlice1,
               *rdata = cpl_table_get_data_float_const(aSamples, "right");
 
   /* plot title */
-  fprintf(gp, "set title \"trace slice widths, slices %hu to %hu\"\n",
-          aSlice1, aSlice2);
+  fprintf(gp, "set title \"trace slice widths, ");
+  if (aIFU > 0) {
+    fprintf(gp, "IFU %hhu, ", aIFU);
+  }
+  fprintf(gp, "slices %hu to %hu\"\n", aSlice1, aSlice2);
   fprintf(gp, "set key outside below\n");
   /* set ranges and axes for plotting */
   fprintf(gp, "set xrange [%d:%d]\n", 1, kMuseOutputYTop);
