@@ -230,8 +230,11 @@ muse_wcs_centroid_stars(muse_image *aImage, float aSigma,
     cpl_error_set(__func__, CPL_ERROR_DATA_NOT_FOUND);
     return NULL;
   }
-  cpl_msg_debug(__func__, "The %.3f sigma threshold was used to find %d sources",
-                aSigma, ndet);
+  double med_dist, median = cpl_image_get_median_dev(image->data, &med_dist),
+         threshold = median + aSigma * med_dist;
+  cpl_msg_debug(__func__, "The %.3f sigma threshold (%.3f, %.3f +/- %.3f) was "
+                "used to find %d sources", aSigma, threshold, median, med_dist,
+                ndet);
 
   int nx = cpl_image_get_size_x(image->data),
       ny = cpl_image_get_size_y(image->data);
@@ -1090,7 +1093,7 @@ muse_wcs_optimize_solution(muse_wcs_object *aWCSObj, float aDetSigma,
   }
 
   /* reject failed results and those far away from the expected scale */
-  double scale = muse_pfits_get_mode(aWCSObj->cube->header) > MUSE_MODE_WFM_AO_N
+  double scale = muse_pfits_get_mode(aWCSObj->cube->header) == MUSE_MODE_NFM_AO_N
                ? kMuseSpaxelSizeX_NFM : kMuseSpaxelSizeY_WFM;
   cpl_msg_debug(__func__, "pruning results +/-10%% away from expected spaxel "
                 "scale of %.3f arcsec/pixel", scale);
