@@ -45,7 +45,7 @@
   Regular expression to copy/erase all keywords relevant to world coordinates
   this list is mostly taken from the FITS Standard v3.0 (Tab. 8.2, p. 77).
  */
-#define MUSE_WCS_KEYS  "^C(TYPE|UNIT|RPIX|RVAL|DELT|SYER)|^CD[0-9]+_[0-9]+|" \
+#define MUSE_WCS_KEYS  "^C(TYPE|UNIT|RPIX|RVAL|DELT|SYER|RDER)|^CD[0-9]+_[0-9]+|" \
                        "^WCSAXES$|^L[OA][NT]POLE$"
 /**
   @brief   name for special astrometry detection image
@@ -263,6 +263,7 @@ void muse_wcs_object_delete(muse_wcs_object *);
 cpl_table *muse_wcs_centroid_stars(muse_image *, float, muse_wcs_centroid_type);
 cpl_error_code muse_wcs_locate_sources(muse_pixtable *, float, muse_wcs_centroid_type, muse_wcs_object *);
 cpl_error_code muse_wcs_solve(muse_wcs_object *, cpl_table *, float, float, int, float);
+cpl_error_code muse_wcs_optimize_solution(muse_wcs_object *, float, muse_wcs_centroid_type, cpl_table *, float, float, int, float);
 cpl_propertylist *muse_wcs_create_default(void);
 cpl_propertylist *muse_wcs_apply_cd(const cpl_propertylist *, const cpl_propertylist *);
 cpl_error_code muse_wcs_project_tan(muse_pixtable *, const cpl_propertylist *);
