@@ -1343,21 +1343,21 @@ muse_align_compute_field_offsets(muse_imagelist *aImagelist,
   cpl_table_subtract_scalar(offsets, MUSE_OFFSETS_DDEC, dec_shift);
   cpl_table_set_column_invalid(offsets, MUSE_OFFSETS_FSCALE, 0, nfields);
 
-  /* Extra QC parameters: offset statistics */
-  double qcmin  = cpl_table_get_column_min(offsets, MUSE_OFFSETS_DRA);
-  double qcmax  = cpl_table_get_column_max(offsets, MUSE_OFFSETS_DRA);
-  double qcmean = cpl_table_get_column_mean(offsets, MUSE_OFFSETS_DRA);
-  double qcsdev = cpl_table_get_column_stdev(offsets, MUSE_OFFSETS_DRA);
+  /* Extra QC parameters: offset statistics, in arcsec */
+  double qcmin  = cpl_table_get_column_min(offsets, MUSE_OFFSETS_DRA) * 3600.;
+  double qcmax  = cpl_table_get_column_max(offsets, MUSE_OFFSETS_DRA) * 3600.;
+  double qcmean = cpl_table_get_column_mean(offsets, MUSE_OFFSETS_DRA) * 3600.;
+  double qcsdev = cpl_table_get_column_stdev(offsets, MUSE_OFFSETS_DRA) * 3600.;
 
   cpl_propertylist_append_double(aHeader, QC_ALIGN_DRA_MIN, qcmin);
   cpl_propertylist_append_double(aHeader, QC_ALIGN_DRA_MAX, qcmax);
   cpl_propertylist_append_double(aHeader, QC_ALIGN_DRA_MEAN, qcmean);
   cpl_propertylist_append_double(aHeader, QC_ALIGN_DRA_STDEV, qcsdev);
 
-  qcmin  = cpl_table_get_column_min(offsets, MUSE_OFFSETS_DDEC);
-  qcmax  = cpl_table_get_column_max(offsets, MUSE_OFFSETS_DDEC);
-  qcmean = cpl_table_get_column_mean(offsets, MUSE_OFFSETS_DDEC);
-  qcsdev = cpl_table_get_column_stdev(offsets, MUSE_OFFSETS_DDEC);
+  qcmin  = cpl_table_get_column_min(offsets, MUSE_OFFSETS_DDEC) * 3600.;
+  qcmax  = cpl_table_get_column_max(offsets, MUSE_OFFSETS_DDEC) * 3600.;
+  qcmean = cpl_table_get_column_mean(offsets, MUSE_OFFSETS_DDEC) * 3600.;
+  qcsdev = cpl_table_get_column_stdev(offsets, MUSE_OFFSETS_DDEC) * 3600.;
 
   cpl_propertylist_append_double(aHeader, QC_ALIGN_DDEC_MIN, qcmin);
   cpl_propertylist_append_double(aHeader, QC_ALIGN_DDEC_MAX, qcmax);
