@@ -278,6 +278,7 @@ muse_lsf_params_save_table(cpl_propertylist *aHeader,
       r = cpl_propertylist_save(aHeader, filename, CPL_IO_CREATE);
       r = muse_lsf_params_save(aSlicePars, filename);
       if (r == CPL_ERROR_NONE) {
+#pragma omp critical(muse_processing_output_frames)
         cpl_frameset_insert(aProcessing->outframes, frame);
       } else {
         cpl_frame_delete(frame);
@@ -307,11 +308,11 @@ muse_lsf_compute(muse_processing *aProcessing, muse_lsf_params_t *aParams)
   muse_basicproc_params_delete(bpars);
   cpl_ensure(images, cpl_error_get_code(), -1);
 
-  cpl_table *arclines = muse_table_load(aProcessing, MUSE_TAG_LINE_CATALOG, 0);
-  cpl_table *tracetable = muse_table_load(aProcessing, MUSE_TAG_TRACE_TABLE,
-                                          aParams->nifu);
-  cpl_table *wavecaltable = muse_table_load(aProcessing, MUSE_TAG_WAVECAL_TABLE,
-                                            aParams->nifu);
+  cpl_table *arclines = muse_processing_load_ctable(aProcessing, MUSE_TAG_LINE_CATALOG, 0);
+  cpl_table *tracetable = muse_processing_load_ctable(aProcessing, MUSE_TAG_TRACE_TABLE,
+                                                      aParams->nifu);
+  cpl_table *wavecaltable = muse_processing_load_ctable(aProcessing, MUSE_TAG_WAVECAL_TABLE,
+                                                        aParams->nifu);
   if (!arclines || !tracetable || !wavecaltable) {
     cpl_msg_error(__func__, "Calibration could not be loaded:%s%s%s",
                   !arclines ? " " : "",
