@@ -113,36 +113,32 @@ muse_scibasic_per_exposure(muse_processing *aProcessing,
                                   aParams->skybinsize, lsigma, hsigma, niter);
   }
 
-  if (aParams->resample) {
-    /* to visualize the current state better than the pixtable *
-     * can do, resample the data onto an image                 */
-    muse_image *image = muse_resampling_image(pixtable,
-                                              MUSE_RESAMPLE_WEIGHTED_RENKA,
-                                              1., aParams->dlambda);
-    char *tag = cpl_sprintf("%s_RESAMPLED", intag);
-    /* don't want to save QC headers with the samples table: */
-    cpl_propertylist_erase_regexp(image->header, QC_SCIBASIC_PREFIX, 0);
-    muse_processing_save_image(aProcessing, aParams->nifu, image, tag);
-    cpl_free(tag);
-    muse_image_delete(image);
-  }
-
   /* Trim the pixel table to a useful wavelength range before saving it. *
    * Use slightly larger ranges to not cut too much, and so leave the    *
    * first plane of the output datacube hopefully mostly intact.         */
-  const double llimN = 4750. - FLT_EPSILON, hlimN = 9350. + FLT_EPSILON,
-               llimE = 4600. - FLT_EPSILON, hlimE = 9350. + FLT_EPSILON;
+  const double llimN = kMuseUsefulNLambdaMin - FLT_EPSILON,
+               llimE = kMuseUsefulELambdaMin - FLT_EPSILON,
+               llimA = kMuseUsefulAOLambdaMin - FLT_EPSILON,
+               hlim = kMuseUsefulLambdaMax + FLT_EPSILON;
+  muse_ins_mode mode = muse_pfits_get_mode(pixtable->header);
   if (aParams->crop) {
-    if (muse_pfits_get_mode(pixtable->header) > MUSE_MODE_WFM_NONAO_X) {
+    if ((mode == MUSE_MODE_WFM_AO_N) || (mode == MUSE_MODE_NFM_AO_N)) {
+      cpl_msg_info(__func__, "Nominal AO mode: cropping the pixel table of IFU "
+                   "%d to %.1f...%.1f Angstrom", aParams->nifu, llimA, hlim);
+      muse_pixtable_restrict_wavelength(pixtable, llimA, hlim);
+    } else if (mode == MUSE_MODE_WFM_NONAO_N) {
       cpl_msg_info(__func__, "Nominal mode: cropping the pixel table of IFU %d"
-                   " to %.1f...%.1f Angstrom", aParams->nifu, llimN, hlimN);
-      muse_pixtable_restrict_wavelength(pixtable, llimN, hlimN);
+                   " to %.1f...%.1f Angstrom", aParams->nifu, llimN, hlim);
+      muse_pixtable_restrict_wavelength(pixtable, llimN, hlim);
     } else { /* extended mode */
       cpl_msg_info(__func__, "Extended mode: cropping the pixel table of IFU %d"
-                   " to %.1f...%.1f Angstrom", aParams->nifu, llimE, hlimE);
-      muse_pixtable_restrict_wavelength(pixtable, llimE, hlimE);
+                   " to %.1f...%.1f Angstrom", aParams->nifu, llimE, hlim);
+      muse_pixtable_restrict_wavelength(pixtable, llimE, hlim);
     }
   } /* if crop */
+  if (mode > MUSE_MODE_WFM_NONAO_N) {
+    muse_basicproc_mask_notch_filter(pixtable, aParams->nifu);
+  } /* if AO mode */
 
   if (aAttached) { /* test before, to not throw an error */
     muse_basicproc_apply_illum(pixtable, aAttached);
@@ -156,6 +152,20 @@ muse_scibasic_per_exposure(muse_processing *aProcessing,
     muse_basicproc_apply_twilight(pixtable, aTwilights[icor]);
   } /* for twilight correction cube not NULL */
 
+  if (aParams->resample) {
+    /* to visualize the current state better than the pixtable *
+     * can do, resample the data onto an image                 */
+    muse_image *image = muse_resampling_image(pixtable,
+                                              MUSE_RESAMPLE_WEIGHTED_RENKA,
+                                              1., aParams->dlambda);
+    char *tag = cpl_sprintf("%s_RESAMPLED", intag);
+    /* don't want to save QC headers with the samples table: */
+    cpl_propertylist_erase_regexp(image->header, QC_SCIBASIC_PREFIX, 0);
+    muse_processing_save_image(aProcessing, aParams->nifu, image, tag);
+    cpl_free(tag);
+    muse_image_delete(image);
+  } /* if resample */
+
   /* construct output tag and finally save */
   char *outtag = cpl_sprintf("PIXTABLE_%s", intag);
   muse_processing_save_table(aProcessing, aParams->nifu, pixtable, NULL, outtag,
@@ -201,7 +211,11 @@ muse_scibasic_compute(muse_processing *aProcessing,
     bpars->flatimage = NULL; /* so that it doesn't get deallocated */
     muse_basicproc_params_delete(bpars);
   }
-  cpl_ensure(images, cpl_error_get_code(), -1);
+  if (!images) {
+    muse_image_delete(masterflat);
+    cpl_error_set(__func__, cpl_error_get_code());
+    return -1;
+  }
 
   cpl_table *tracetable = muse_processing_load_ctable(aProcessing, MUSE_TAG_TRACE_TABLE,
                                                       aParams->nifu);
