@@ -76,6 +76,9 @@ muse_scipost_calibrate_flux_compute(muse_processing *aProcessing,
     muse_processing_append_used(aProcessing, frame, CPL_FRAME_GROUP_RAW, 1);
     /* erase pre-existing QC parameters */
     cpl_propertylist_erase_regexp(pixtable->header, "ESO QC ", 0);
+    /* See if we need to revert the flat-field spectrum correction, *
+     * to match pixel table and response curve, before applying it. */
+    muse_postproc_revert_ffspec_maybe(pixtable, response);
     rc = muse_flux_calibrate(pixtable, response, extinction, telluric);
     if (rc != CPL_ERROR_NONE) {
       cpl_msg_error(__func__, "while muse_flux_calibrate(%s)", 
