@@ -79,9 +79,7 @@ muse_scipost_make_cube_compute(muse_processing *aProcessing,
   rp->crsigma = aParams->crsigma;
   rp->ld = aParams->ld;
   rp->rc = aParams->rc;
-  rp->pfx = aParams->pixfrac;
-  rp->pfy = aParams->pixfrac;
-  rp->pfl = aParams->pixfrac;
+  muse_resampling_params_set_pixfrac(rp, aParams->pixfrac);
   cpl_propertylist *outwcs = muse_postproc_cube_load_output_wcs(aProcessing);
   muse_resampling_params_set_wcs(rp, outwcs);
   cpl_propertylist_delete(outwcs);
