@@ -110,9 +110,10 @@ muse_standard_compute(muse_processing *aProcessing,
     muse_postproc_properties_delete(prop);
     return -1;
   }
-  prop->extinction = muse_table_load(aProcessing, MUSE_TAG_EXTINCT_TABLE, 0);
+  prop->extinction = muse_processing_load_ctable(aProcessing,
+                                                 MUSE_TAG_EXTINCT_TABLE, 0);
   /* telluric regions */
-  prop->tellregions = muse_table_load(aProcessing, MUSE_TAG_TELLREG, 0);
+  prop->tellregions = muse_processing_load_ctable(aProcessing, MUSE_TAG_TELLREG, 0);
   if (!prop->tellregions) {
     cpl_msg_debug(__func__, "%s could not be found or loaded, using defaults",
                   MUSE_TAG_TELLREG);
@@ -152,11 +153,12 @@ muse_standard_compute(muse_processing *aProcessing,
     int ifilt, ipos = 0, nfilt = cpl_array_get_size(filters);
     for (ifilt = 0; ifilt < nfilt; ifilt++) {
       /* try to find and load the filter from a table */
-      cpl_table *filter = muse_table_load_filter(aProcessing,
-                                                 cpl_array_get_string(filters, ifilt));
+      const char *filtername = cpl_array_get_string(filters, ifilt);
+      muse_table *filter = muse_table_load_filter(aProcessing, filtername);
       if (!filter) {
         continue;
       }
+      /* add an image integrated over this filter to the output cube */
       muse_image *fov = muse_datacube_collapse(fluxobjs[i]->cube, filter);
       if (!fluxobjs[i]->cube->recimages) {
         fluxobjs[i]->cube->recimages = muse_imagelist_new();
@@ -166,7 +168,15 @@ muse_standard_compute(muse_processing *aProcessing,
       cpl_array_set_size(fluxobjs[i]->cube->recnames, ipos+1);
       cpl_array_set_string(fluxobjs[i]->cube->recnames, ipos,
                            cpl_array_get_string(filters, ifilt));
-      cpl_table_delete(filter);
+      /* check that the filter we are processing is one for which *
+       * we actually want a zeropoint measurement before calling  */
+      if (!strncmp(filtername, "Johnson_V", 10) ||
+          !strncmp(filtername, "Cousins_R", 10) ||
+          !strncmp(filtername, "Cousins_I", 10)) {
+        /* add zeropoint to the QC parameters, using this filter */
+        muse_flux_compute_qc_zp(fluxobjs[i], filter, filtername);
+      } /* if matching filter */
+      muse_table_delete(filter);
       ipos++;
     } /* for ifilt (all filters) */
     muse_postproc_qc_fwhm(aProcessing, fluxobjs[i]->cube); /* before NANs! */
@@ -187,7 +197,7 @@ muse_standard_compute(muse_processing *aProcessing,
                                                         fluxobjs[i]->response,
                                                         header,
                                                         MUSE_TAG_STD_RESPONSE,
-                                                        MUSE_TABLE_TYPE_CPL);
+                                                        MUSE_TABLE_TYPE_MUSE);
     cpl_free(object);
     if (rc != CPL_ERROR_NONE) {
       cpl_free(objorig);
@@ -200,7 +210,7 @@ muse_standard_compute(muse_processing *aProcessing,
     object = cpl_sprintf("Telluric correction (%s)", objorig);
     cpl_propertylist_update_string(header, "OBJECT", object);
     rc = muse_processing_save_table(aProcessing, -1, fluxobjs[i]->telluric, header,
-                                    MUSE_TAG_STD_TELLURIC, MUSE_TABLE_TYPE_CPL);
+                                    MUSE_TAG_STD_TELLURIC, MUSE_TABLE_TYPE_MUSE);
     cpl_free(object);
     cpl_free(objorig);
     cpl_propertylist_delete(header);
