@@ -208,6 +208,11 @@ muse_twilight_reconstruct(muse_processing *aProcessing,
     } /* if pixel table created */
     muse_image_delete(masterimage);
     cpl_table_delete(tattached);
+
+    /* in AO mode we also need to mask the NaD notch filter range */
+    if (muse_pfits_get_mode(pts[nifu - 1]->header) > MUSE_MODE_WFM_NONAO_N) {
+      muse_basicproc_mask_notch_filter(pts[nifu - 1], nifu);
+    } /* if AO mode */
   } /* for nifu (all IFUs) */
   cpl_table_delete(geo);
 
