@@ -191,7 +191,7 @@ muse_wavecal_compute(muse_processing *aProcessing,
    * by copying most of the input headers.                               */
   cpl_table *wavecaltable = NULL;
   cpl_propertylist *header = NULL;
-  muse_wave_params *p = muse_wave_params_new();
+  muse_wave_params *p = muse_wave_params_new(muse_imagelist_get(images, 0)->header);
   p->xorder = aParams->xorder;
   p->yorder = aParams->yorder;
   p->detsigma = aParams->sigma;
