@@ -33,7 +33,7 @@ def crea_mask(image,min,max):
         vector = image[~where_are_NaNs]
         num = len(vector)
         vector.sort()
-        sel = vector[num*min : num*max]
+        sel = vector[int(num*min) : int(num*max)]
         min_sel = sel.min()
         max_sel = sel.max()
 
