@@ -1299,110 +1299,182 @@ if __name__ == '__main__':
   files=in_sof.files
   files_svd=in_svd.files
   param  = inputs.in_sop
+
  
-  a = str.split(param)
+  params  = inputs.in_sop
+  values= {}
+  for param in params:
+          values[param.name]= param.value
 
-  
+  zlevel=  values['muse_zap.zlevel']
+  cftype=  values['muse_zap.cftype']       
+  cfwidthSVD= values['muse_zap.cfwidthSVD']   
+  cfwidthSP= values['muse_zap.cfwidthSP']    
+  optimizeType= values['muse_zap.optimizeType'] 
+  pevals= values['muse_zap.pevals']       
+  nevals= values['muse_zap.nevals']  
+  clean= values['muse_zap.clean']
+  extSVD= values['muse_zap.extSVD']
 
-  exec(a[0]) #zlevel
-  exec(a[1]) #cftype
-  exec(a[2]) #cfwidthSVD
-  exec(a[3]) #cfwidthSP
-  exec(a[4]) #optimizeType
-  exec(a[5]) #pevals
-  exec(a[6]) #nevals
-  exec(a[7]) #clean
-  exec(a[8])#extSVD
   pattern = '--products-dir'
   for arg in sys.argv:
      if arg.split("=")[0] == pattern:
          output_dir = arg.split("=")[1]
   mask_filename=None
+  IsThereSkyCube = 0 
+  IsThereObjectCube = 0
+  IsThereSVD = 0 
+  
   for file in files:
       if file.category == 'MASK_FINAL_CUBE':
          mask_filename = file.name
-         
+      if file.category == 'DATACUBE_SKY':
+         IsThereSkyCube = 1 
+         musecubefits_filename = file.name 
+         output_purpose=file.purposes
+         output_datasetname=in_sof.datasetName
+      if file.category == 'DATACUBE_OBJECT':
+         IsThereObjectCube = 1
+         musecubefits_filename = file.name
+         output_purpose=file.purposes
+         output_datasetname=in_sof.datasetName
+      if file.category == 'ZAP_SVD':
+         IsThereSVD=1
+         if extSVD == 'none' or extSVD == 'None' or extSVD == 'NONE' or extSVD == None:
+            extSVD = file.name
+      if extSVD == 'none' or extSVD == 'None' or extSVD == 'NONE' or extSVD == None:
+          extSVD=None
+          svdoutputfits_filename = output_dir+ '/ZAP_SVD.fits'
+ 
+  if IsThereObjectCube == 1 and IsThereSkyCube ==1:
+      error
 
-  for file in files:
-      if file.category == 'DATACUBE_FINAL':
-        musecubefits_filename = file.name
-        output_purpose=file.purposes
-        output_datasetname=in_sof.datasetName
+  #Processing Skyresidual cube to get SVD from
+  if IsThereSkyCube == 1:   
+     svdoutputfits_filename = output_dir+ '/ZAP_SVD.fits'
+     logger.info(svdoutputfits_filename)
+     SVDoutput(musecubefits_filename, svdoutputfits=svdoutputfits_filename,
+                  clean=(clean=='True'), zlevel=zlevel, cftype=cftype,
+                  cfwidth=cfwidthSVD, mask=mask_filename)
+ 
+     hdu2 = fits.open(svdoutputfits_filename,method='update')
+     hdu =  fits.open(musecubefits_filename)
+     hdu2[0].header = hdu[0].header
+     hdu2[0].header['HIERARCH ESO SKY ORIG'] =  musecubefits_filename.split("/")[-1]
+     hdu2[0].header['HIERARCH ESO PRO CATG'] = 'ZAP_SVD'
 
-        file.name = output_dir+ '/DATACUBE_CLEANED.fits'
-        file.category = 'DATACUBE_CLEANED'
-        outcubefits_filename = file.name
-        
+     i=0
+     for param in params:
+       i=i+1
+       hkey1 = 'HIERARCH ESO ZAP REC'+str(i)+' NAME'
+       hkey2 = 'HIERARCH ESO ZAP REC'+str(i)+' VALUE'
+       hdu2[0].header[hkey1] = param.name
+       hdu2[0].header[hkey2] = param.value
+     hdu2.writeto(svdoutputfits_filename,clobber=True)
 
-        if optimizeType != 'none':
-          pevals=[]
-          nevals=[]
-          
-        if extSVD == 'none':
-           extSVD=None
-           svdoutputfits_filename = output_dir+ '/ZAP_SVD.fits'
-        else:
-           mask_filename=None
-           svdoutputfits_filename=None
+     # create output SOF:
+     files = list()
+     output_category = ['ZAP_SVD']
+     output_name = [svdoutputfits_filename]
+     files.append(reflex.FitsFile(output_name[0],output_category[0],None,output_purpose))
+     newsof = reflex.SetOfFiles(output_datasetname,files)
+     outputs.out_sof = newsof
+
+
+  #Processing Object cube
+  if IsThereObjectCube == 1:
+     outcubefits_filename = output_dir+ '/DATACUBE_CLEANED.fits'
+     svdoutputfits_filename = output_dir+ '/ZAP_SVD.fits' 
+     output_category = ['DATACUBE_CLEANED','ZAP_SVD']
+     if extSVD == 'none' or extSVD == 'None' or extSVD == 'NONE' or extSVD == None:
+        extSVD=None
+     else:   #extSVD is provided, either via the parameter name or via the sof (see above)       
+        mask_filename=None
+        svdoutputfits_filename=None
+     if optimizeType != 'none':
+        pevals=[]
+        nevals=[]
      
-        #if skycubefits == 'none':
-        #   skycubefits=None 
-
-         
-        try:
-          
-          if pevals <= 0:
-            process(musecubefits_filename,outcubefits=outcubefits_filename,clean=(clean=='True'),zlevel=zlevel,cftype=cftype, cfwidthSVD=cfwidthSVD, cfwidthSP=cfwidthSP,  nevals=nevals, optimizeType=optimizeType, extSVD=extSVD, skycubefits=None, svdoutputfits=svdoutputfits_filename, mask=mask_filename, interactive='False')
-          else:
-            #process(bla)
+     try:          
+        if pevals <= 0:
+           process(musecubefits_filename,outcubefits=outcubefits_filename,clean=(clean=='True'),zlevel=zlevel,cftype=cftype, cfwidthSVD=cfwidthSVD, cfwidthSP=cfwidthSP,  nevals=nevals, optimizeType=optimizeType, extSVD=extSVD, skycubefits=None, svdoutputfits=svdoutputfits_filename, mask=mask_filename, interactive='False')
+        else:
+           
            process(musecubefits_filename,outcubefits=outcubefits_filename,clean=(clean=='True'),zlevel=zlevel,cftype=cftype, cfwidthSVD=cfwidthSVD, cfwidthSP=cfwidthSP, pevals=pevals, optimizeType=optimizeType, extSVD=extSVD, skycubefits=None, svdoutputfits=svdoutputfits_filename, mask=mask_filename, interactive='False')
            
-        except: 
+     except: 
+          logger.info(' execution error, check if products already exist')
           outputs.errors = traceback.format_exc() 
           parser.write_outputs()
           #raise Exception('help: I am out of memory')
           sys.exit()
- 
 
+     if extSVD != None:
+        logger.info(' provided')
+        hdu2 =  fits.open(extSVD,method='update')
+        svdoutputfits_filename = extSVD
+     else:
+        logger.info(' created')
+        hdu2 =  fits.open(svdoutputfits_filename,method='update')
+     # create output SOF:
+     files = list()
 
-      
-  hdu =  fits.open(musecubefits_filename)
-  #
-  # create output SOF:
-  files = list()
-  output_category = ['DATACUBE_CLEANED','ZAP_SVD']
-  svdoutputfits_filename = output_dir+ '/ZAP_SVD.fits'
-  output_name = [outcubefits_filename,svdoutputfits_filename]
-  # if I used an input SVD
-  logger.info('ok')
-  if extSVD != None:
-     logger.info(' provided')
-     hdu2 =  fits.open(extSVD,method='update')
-  else:
-     logger.info(' created')
-     hdu2 =  fits.open(svdoutputfits_filename,method='update')
-  files.append(reflex.FitsFile(output_name[0],output_category[0],None,output_purpose))
-  files.append(reflex.FitsFile(output_name[1],output_category[1],None,output_purpose))
-
-  newsof = reflex.SetOfFiles(output_datasetname,files)
-  outputs.out_sof = newsof
-
+     # If I do not have already an input SVD, I broadcast the one I computed
+     if IsThereSVD == 0:
+         output_name = [outcubefits_filename ,svdoutputfits_filename]
+         output_category = ['DATACUBE_CLEANED','ZAP_SVD']
+         files.append(reflex.FitsFile(output_name[0],output_category[0],None,output_purpose))
+         files.append(reflex.FitsFile(output_name[1],output_category[1],None,output_purpose))
+         
+         hdu =  fits.open(musecubefits_filename)
+         hdu2[0].header = hdu[0].header
+         hdu2[0].header['OBJECT'] = hdu[0].header['HIERARCH ESO OBS TARG NAME']
+         hdu2[0].header['HIERARCH ESO PRO CATG'] =  'ZAP_SVD'
+         
+         hdu3 = fits.open(outcubefits_filename,method='update')
+         hdu3[0].header['HIERARCH ESO PRO CATG'] = 'DATACUBE_CLEANED'
+         hdu3[0].header['HIERARCH ESO SVD ORIG'] = 'none'
     
-  hdu2[0].header = hdu[0].header
-  hdu2[0].header['OBJECT'] = hdu[0].header['HIERARCH ESO OBS TARG NAME']
-  hdu2[0].header['HIERARCH ESO PRO CATG'] =  'ZAP_SVD'
+         i=0
+         for param in params:
+             i=i+1
+             hkey1 = 'HIERARCH ESO ZAP REC'+str(i)+' NAME'
+             hkey2 = 'HIERARCH ESO ZAP REC'+str(i)+' VALUE'
+             hdu2[0].header[hkey1] = param.name
+             hdu2[0].header[hkey2] = param.value
+             hdu3[0].header[hkey1] = param.name
+             hdu3[0].header[hkey2] = param.value
 
-  hdu3 = fits.open(outcubefits_filename,method='update')
-  hdu3[0].header['HIERARCH ESO PRO CATG'] = 'DATACUBE_CLEANED'
+             
+         hdu2.writeto(svdoutputfits_filename,clobber=True)
+         hdu3.writeto(outcubefits_filename,clobber=True)
+         
+         newsof = reflex.SetOfFiles(output_datasetname,files)
+         outputs.out_sof = newsof
+    # If I have already an input SVD, I do not broadcast it.
+     else:
+         logger.info('doing this')
+         output_name = [outcubefits_filename]
+         output_category = ['DATACUBE_CLEANED']
+         files.append(reflex.FitsFile(output_name[0],output_category[0],None,output_purpose))
+         
+         hdu3 = fits.open(outcubefits_filename,method='update')
+         hdu3[0].header['HIERARCH ESO PRO CATG'] = 'DATACUBE_CLEANED'
+         hdu3[0].header['HIERARCH ESO SVD ORIG'] =  extSVD.split("/")[-1]
+    
+         i=0
+         for param in params:
+             i=i+1
+             hkey1 = 'HIERARCH ESO ZAP REC'+str(i)+' NAME'
+             hkey2 = 'HIERARCH ESO ZAP REC'+str(i)+' VALUE'
+             hdu3[0].header[hkey1] = param.name
+             hdu3[0].header[hkey2] = param.value
 
-
-  #Note: a new file will be created for the input SVD in any case, To update the header.
-  hdu2.writeto(svdoutputfits_filename,clobber=True)
-  hdu3.writeto(outcubefits_filename,clobber=True)
+         hdu3.writeto(outcubefits_filename,clobber=True)
+         newsof = reflex.SetOfFiles(output_datasetname,files)
+         outputs.out_sof = newsof
 
 # emitt output:
   parser.write_outputs()
   sys.exit()
  
-
-
