@@ -9,7 +9,7 @@ try:
     #import_ok = 1
 except ImportError:
     try:
-       from astropy.io import pyfits as pyfits
+       from astropy.io import fits as pyfits
      #  import_ok = 1
     except ImportError:
        import_ok = 0
