@@ -2,7 +2,7 @@
 /* vim:set sw=2 sts=2 et cin: */
 /*
  * This file is part of the MUSE Instrument Pipeline
- * Copyright (C) 2007-2015 European Southern Observatory
+ * Copyright (C) 2007-2017 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -363,10 +363,8 @@ muse_test_flux_integrate_from_cube(int argc, char **argv)
   fo->intimage = intimage;
   fo->raref = muse_pfits_get_ra(cube->header);
   fo->decref = muse_pfits_get_dec(cube->header);
-  cpl_test_null(fo->reference);
   muse_flux_response_compute(fo, select, airmass, reftable,
                              NULL, exttable);
-  cpl_test_nonnull(fo->reference);
   muse_flux_get_response_table(fo, smooth);
   muse_flux_compute_qc(fo);
   fn = cpl_sprintf("%s_STD_RESPONSE.fits", incube);
@@ -382,8 +380,7 @@ muse_test_flux_integrate_from_cube(int argc, char **argv)
   cpl_table_delete(reftable);
   cpl_table_delete(exttable);
 
-  cpl_test(cpl_errorstate_is_equal(state));
-  return CPL_ERROR_NONE;
+  return cpl_errorstate_is_equal(state) ? CPL_ERROR_NONE : cpl_error_get_code();
 } /* muse_test_flux_integrate_from_cube() */
 
 /*----------------------------------------------------------------------------*/
@@ -408,14 +405,17 @@ muse_test_flux_integrate_from_cube(int argc, char **argv)
 /*----------------------------------------------------------------------------*/
 int main(int argc, char **argv)
 {
-  cpl_test_init(PACKAGE_BUGREPORT, CPL_MSG_DEBUG);
-
+  cpl_error_code rc = CPL_ERROR_NONE;
   if (argc > 1) {
     /* If there are arguments, misuse this test program as flux integrator. */
-    cpl_test(muse_test_flux_integrate_from_cube(argc, argv) == CPL_ERROR_NONE);
-    return cpl_test_end(0);
+    cpl_init(CPL_INIT_DEFAULT);
+    rc = muse_test_flux_integrate_from_cube(argc, argv);
+    cpl_end();
+    return rc;
   }
 
+  /* start as test program */
+  cpl_test_init(PACKAGE_BUGREPORT, CPL_MSG_DEBUG);
   cpl_table *ref = cpl_table_load(BASEFILENAME"_reference_gd71sm.fits", 1, 1),
             *extinction = cpl_table_load(BASEFILENAME"_extinction.fits", 1, 1);
 
@@ -502,7 +502,7 @@ int main(int argc, char **argv)
   cpl_test_nonnull(fluxobj);
   /* flux integration using Moffat profile fit */
   state = cpl_errorstate_get();
-  cpl_error_code rc = muse_flux_integrate_std(pt, MUSE_FLUX_PROFILE_MOFFAT, fluxobj);
+  rc = muse_flux_integrate_std(pt, MUSE_FLUX_PROFILE_MOFFAT, fluxobj);
   cpl_test(rc == CPL_ERROR_NONE && cpl_errorstate_is_equal(state));
   cpl_test_nonnull(fluxobj->cube);
   cpl_test_nonnull(fluxobj->intimage);
@@ -578,6 +578,71 @@ int main(int argc, char **argv)
       muse_datacube_delete(cube2);
     } /* if MUSE_FLUX_PROFILE_GAUSSIAN */
   } /* for i (all other integration profiles) */
+
+  /* test with pixel table with the other instrument modes */
+  const char *mode[] = { "WFM-NOAO-E", "WFM-AO-N", "WFM-AO-E", "WFM-AO-N",
+                         "NFM-AO-N", NULL };
+  for (i = 0; mode[i] != NULL; i++) {
+    muse_pixtable *pt3 = muse_pixtable_duplicate(pt);
+    /* set mode */
+    cpl_propertylist_update_string(pt3->header, "ESO INS MODE", mode[i]);
+    cpl_msg_debug(__func__, "original mode: %s, updated mode: %s",
+                  muse_pfits_get_insmode(pt->header),
+                  muse_pfits_get_insmode(pt3->header));
+    muse_flux_object *fobj3 = muse_flux_object_new();
+    state = cpl_errorstate_get();
+    rc = muse_flux_integrate_std(pt3, MUSE_FLUX_PROFILE_MOFFAT, fobj3);
+    cpl_test(rc == CPL_ERROR_NONE && cpl_errorstate_is_equal(state));
+    cpl_test_nonnull(fobj3->cube);
+#if 0
+    char *fn3 = cpl_sprintf("cube_%s_3.fits", mode[i]);
+    muse_datacube_save(fobj3->cube, fn3);
+    cpl_free(fn3);
+#endif
+    cpl_test_nonnull(fobj3->intimage);
+    cpl_test(cpl_imagelist_get_size(fobj3->cube->data)
+             == cpl_image_get_size_x(fobj3->intimage->data));
+    if (!strstr(mode[i], "NFM")) {
+      /* should give identical integration image as original, if not NFM */
+      cpl_test_image_rel(fobj3->intimage->data, fluxobj->intimage->data,
+                         FLT_EPSILON);
+      cpl_test_image_rel(fobj3->intimage->stat, fluxobj->intimage->stat,
+                         FLT_EPSILON);
+    }
+    /* done for this mode, if this is data pretending to be non-AO */
+    if (strstr(mode[i], "NOAO")) {
+      muse_flux_object_delete(fobj3);
+      muse_pixtable_delete(pt3);
+      continue;
+    }
+
+    /* for the AO modes, also test with pixel table containing EURO3D_NOTCH_NAD */
+    muse_basicproc_mask_notch_filter(pt3, 3); /* hack! */
+    cpl_table_select_all(pt3->table);
+    muse_flux_object *fobj4 = muse_flux_object_new();
+    state = cpl_errorstate_get();
+    rc = muse_flux_integrate_std(pt3, MUSE_FLUX_PROFILE_MOFFAT, fobj4);
+    cpl_test(rc == CPL_ERROR_NONE && cpl_errorstate_is_equal(state));
+    cpl_test_nonnull(fobj4->cube);
+#if 0
+    char *fn4 = cpl_sprintf("cube_%s_4.fits", mode[i]);
+    muse_datacube_save(fobj4->cube, fn4);
+    cpl_free(fn4);
+#endif
+    cpl_test_nonnull(fobj4->intimage);
+    cpl_test(cpl_imagelist_get_size(fobj4->cube->data)
+             == cpl_image_get_size_x(fobj4->intimage->data));
+    if (!strstr(mode[i], "NFM")) {
+      cpl_test_image_rel(fobj4->intimage->data, fobj3->intimage->data,
+                         FLT_EPSILON);
+      cpl_test_image_rel(fobj4->intimage->stat, fobj3->intimage->stat,
+                         FLT_EPSILON);
+    }
+    muse_flux_object_delete(fobj4);
+    muse_flux_object_delete(fobj3);
+    muse_pixtable_delete(pt3);
+  } /* for i (other instrument modes) */
+
   /* test failure cases of muse_flux_integrate_std(), too */
   muse_flux_object *fobj3 = muse_flux_object_new();
   state = cpl_errorstate_get();
@@ -654,7 +719,9 @@ int main(int argc, char **argv)
   fluxobj->sensitivity = stmp;
 
   /* check muse_flux_get_response_table() */
+  state = cpl_errorstate_get();
   rc = muse_flux_get_response_table(fluxobj, MUSE_FLUX_SMOOTH_MEDIAN);
+  cpl_test(rc == CPL_ERROR_NONE && cpl_errorstate_is_equal(state));
   cpl_test_nonnull(fluxobj->response);
   cpl_test_nonnull(fluxobj->response->table);
   cpl_test_nonnull(fluxobj->response->header);
@@ -671,10 +738,81 @@ int main(int argc, char **argv)
                                              MUSE_FLUX_RESP_FLUX),
          r3 = muse_flux_response_interpolate(fluxobj->response->table, 9000., NULL,
                                              MUSE_FLUX_RESP_FLUX);
-  cpl_msg_debug(__func__, "responses computed: %f ... %f ... %f", r1, r2, r3);
-  cpl_test(fabs(r1 - 41.676) < 0.02 &&
-           fabs(r2 - 41.581) < 0.02 &&
-           fabs(r3 - 41.422) < 0.02);
+  cpl_msg_debug(__func__, "responses computed (WFM-NOAO-N, median): %f...%f..."
+                "%f", r1, r2, r3);
+  cpl_test_abs(r1, 41.676, 0.02);
+  cpl_test_abs(r2, 41.581, 0.02);
+  cpl_test_abs(r3, 41.422, 0.02);
+  /* test with pixel table with the all instrument modes; here, *
+   * also use non-AO nominal mode again, since the above test   *
+   * was without flat-field correction flat in the header       */
+  const char *modeall[] = { "WFM-NOAO-N", "WFM-NOAO-E", "WFM-AO-N", "WFM-AO-E",
+                            "WFM-AO-N", "NFM-AO-N", NULL };
+  for (i = 0; modeall[i] != NULL; i++) {
+    /* "duplicate" the flux object, but really only copy cube header *
+     * and sensitivity; do that twice, one for each smoothing type   */
+    muse_flux_object *fobj3p = muse_flux_object_new(),
+                     *fobj4p = muse_flux_object_new();
+    fobj3p->cube = cpl_calloc(1, sizeof(muse_datacube));
+    fobj3p->cube->header = cpl_propertylist_duplicate(fluxobj->cube->header);
+    fobj3p->sensitivity = cpl_table_duplicate(fluxobj->sensitivity);
+    fobj4p->cube = cpl_calloc(1, sizeof(muse_datacube));
+    fobj4p->cube->header = cpl_propertylist_duplicate(fluxobj->cube->header);
+    fobj4p->sensitivity = cpl_table_duplicate(fluxobj->sensitivity);
+    /* set mode, muse_flux_get_response_table() needs it in the cube header */
+    cpl_propertylist_update_string(fobj3p->cube->header, "ESO INS MODE",
+                                   modeall[i]);
+    cpl_propertylist_update_string(fobj4p->cube->header, "ESO INS MODE",
+                                   modeall[i]);
+    /* also set flat-field corrected flag */
+    cpl_propertylist_update_bool(fobj3p->cube->header, MUSE_HDR_FLUX_FFCORR,
+                                 CPL_TRUE);
+    cpl_propertylist_update_bool(fobj4p->cube->header, MUSE_HDR_FLUX_FFCORR,
+                                 CPL_TRUE);
+
+    state = cpl_errorstate_get();
+    rc = muse_flux_get_response_table(fobj3p, MUSE_FLUX_SMOOTH_MEDIAN);
+    cpl_test(rc == CPL_ERROR_NONE && cpl_errorstate_is_equal(state));
+    cpl_test_nonnull(fobj3p->response);
+    cpl_test_nonnull(fobj3p->response->table);
+    cpl_test_nonnull(fobj3p->response->header);
+    /* the response values should not have changed */
+    double rr1 = muse_flux_response_interpolate(fobj3p->response->table, 5000.,
+                                                NULL, MUSE_FLUX_RESP_FLUX),
+           rr2 = muse_flux_response_interpolate(fobj3p->response->table, 7000.,
+                                                NULL, MUSE_FLUX_RESP_FLUX),
+           rr3 = muse_flux_response_interpolate(fobj3p->response->table, 9000.,
+                                                NULL, MUSE_FLUX_RESP_FLUX);
+    cpl_msg_debug(__func__, "responses computed (%s, median): %f...%f...%f",
+                  muse_pfits_get_insmode(fobj3p->cube->header), rr1, rr2, rr3);
+    cpl_test_abs(r1, rr1, FLT_EPSILON);
+    cpl_test_abs(r2, rr2, FLT_EPSILON);
+    cpl_test_abs(r3, rr3, FLT_EPSILON);
+    cpl_test_abs(rr1, 41.676, 0.02);
+    cpl_test_abs(rr2, 41.581, 0.02);
+    cpl_test_abs(rr3, 41.422, 0.02);
+    muse_flux_object_delete(fobj3p);
+
+    state = cpl_errorstate_get();
+    rc = muse_flux_get_response_table(fobj4p, MUSE_FLUX_SMOOTH_PPOLY);
+    cpl_test(rc == CPL_ERROR_NONE && cpl_errorstate_is_equal(state));
+    cpl_test_nonnull(fobj4p->response);
+    cpl_test_nonnull(fobj4p->response->table);
+    cpl_test_nonnull(fobj4p->response->header);
+    /* again, approximately the same output response */
+    rr1 = muse_flux_response_interpolate(fobj4p->response->table, 5000., NULL,
+                                         MUSE_FLUX_RESP_FLUX),
+    rr2 = muse_flux_response_interpolate(fobj4p->response->table, 7000., NULL,
+                                         MUSE_FLUX_RESP_FLUX),
+    rr3 = muse_flux_response_interpolate(fobj4p->response->table, 9000., NULL,
+                                         MUSE_FLUX_RESP_FLUX);
+    cpl_msg_debug(__func__, "responses computed (%s, ppoly): %f...%f...%f",
+                  muse_pfits_get_insmode(fobj4p->cube->header), rr1, rr2, rr3);
+    cpl_test_abs(rr1, 41.676, 0.02);
+    cpl_test_abs(rr2, 41.581, 0.02);
+    cpl_test_abs(rr3, 41.422, 0.02);
+    muse_flux_object_delete(fobj4p);
+  } /* for i (other instrument modes) */
 
   /* test muse_flux_compute_qc() for the same dataset, now we can check *
    * that we get at least similar values in the output header as the    *
@@ -701,6 +839,8 @@ int main(int argc, char **argv)
   /* special case: OBS.TARG.NAME missing from cube header */
   char *obstarg = cpl_strdup(muse_pfits_get_targname(fluxobj->cube->header));
   cpl_propertylist_erase(fluxobj->cube->header, "ESO OBS TARG NAME");
+  /* check that it works when deleting the new clumn again */
+  cpl_table_erase_column(fluxobj->response->table, "throughput");
   state = cpl_errorstate_get();
   rc = muse_flux_compute_qc(fluxobj);
   cpl_test(cpl_errorstate_is_equal(state) && rc == CPL_ERROR_NONE);
@@ -798,6 +938,8 @@ int main(int argc, char **argv)
   cpl_test(rc == CPL_ERROR_NONE && cpl_errorstate_is_equal(state));
   cpl_test_nonnull(fluxobj->reference);
   cpl_test_nonnull(fluxobj->sensitivity);
+  /* sens1 has the throughput column from QC, so delete it first */
+  cpl_table_erase_column(sens1, "throughput");
   cpl_test_zero(cpl_table_compare_structure(sens1, fluxobj->sensitivity));
   /* compare both tables row by row */
   int nrows1 = cpl_table_get_nrow(sens1),
