@@ -26,7 +26,7 @@
 /*----------------------------------------------------------------------------*
  *                              Includes                                      *
  *----------------------------------------------------------------------------*/
-#define _BSD_SOURCE /* get mkdtemp() from stdlib.h */
+#define _BSD_SOURCE /* get mkdtemp() and setenv() from stdlib.h */
 #include <unistd.h> /* access() */
 #include <stdlib.h>
 #include <stdio.h>
@@ -53,21 +53,16 @@
 
 #define DIR_TEMPLATE "/tmp/muse_test_processing_XXXXXX"
 
-/*----------------------------------------------------------------------------*/
-/**
-  @brief    Prepare the test, mostly by creating a test image with some typical
-            MUSE FITS headers.
-  @param    aDir   the output directory where to save the image
- */
-/*----------------------------------------------------------------------------*/
 static void
-prepare_test(char *aDir)
+muse_test_processing_prepare_test(char *aDir)
 {
+  // create (raw) MUSE-like image
   cpl_image *raw = cpl_image_fill_test_create(4160, 4164);
   /* offset background, so that not everything is marked as saturated */
   cpl_image_add_scalar(raw, 10.);
   /* extension header */
   cpl_propertylist *hdr = cpl_propertylist_new();
+  cpl_propertylist_append_string(hdr, "DATE-OBS", "2016-09-14T12:30:00.001");
   cpl_propertylist_append_int(hdr, "DUMMY", 1);
   cpl_propertylist_append_int(hdr, "ESO DET BINX", 1);
   cpl_propertylist_append_int(hdr, "ESO DET BINY", 1);
@@ -123,10 +118,43 @@ prepare_test(char *aDir)
   cpl_free(file);
   cpl_image_delete(raw);
   cpl_propertylist_delete(hdr);
-} /* prepare_test() */
+
+  // create generic table
+  cpl_table *table = cpl_table_new(5);
+  cpl_table_new_column(table, "column", CPL_TYPE_DOUBLE);
+  cpl_table_fill_column_window(table, "column", 0, 5, DBL_EPSILON);
+  file = cpl_sprintf("%s/table.fits", aDir);
+  cpl_table_save(table, NULL, NULL, file, CPL_IO_CREATE);
+  cpl_free(file);
+  cpl_table_delete(table);
+
+  // create an IFU-specific table (IFU 1)
+  table = cpl_table_new(10);
+  cpl_table_new_column(table, "IFU", CPL_TYPE_INT);
+  cpl_table_fill_column_window(table, "IFU", 0, 9, 1);
+  cpl_propertylist *header = cpl_propertylist_new();
+  cpl_propertylist_append_string(header, "EXTNAME", "CHAN01");
+  cpl_propertylist_append_string(header, "PIPEFILE", "tableifu.fits");
+  file = cpl_sprintf("%s/tableifu.fits", aDir);
+  /* save with header in primary data unit */
+  cpl_table_save(table, header, NULL, file, CPL_IO_CREATE);
+  cpl_free(file);
+  cpl_table_delete(table);
+  cpl_propertylist_delete(header);
+
+  // create a basic FITS header
+  header = cpl_propertylist_new();
+  cpl_propertylist_append_string(header, "INSTRUME", "MUSE");
+  cpl_propertylist_append_double(header, "VALUE", 99.99);
+  file = cpl_sprintf("%s/header.fits", aDir);
+  cpl_propertylist_save(header, file, CPL_IO_CREATE);
+  cpl_free(file);
+  cpl_propertylist_delete(header);
+} /* muse_test_processing_prepare_test() */
 
 static int
-frames_sort_valid_ifus_in_exposure(cpl_table *aTable, int aExposure)
+muse_test_processing_frames_sort_valid_ifus_in_exposure(cpl_table *aTable,
+                                                        int aExposure)
 {
   /* loop through all IFUs and count the number of valid IFUs found */
   int i, nvalid = 0;
@@ -138,10 +166,10 @@ frames_sort_valid_ifus_in_exposure(cpl_table *aTable, int aExposure)
     cpl_free(colname);
   }
   return nvalid;
-} /* frames_sort_valid_ifus_in_exposure() */
+} /* muse_test_processing_frames_sort_valid_ifus_in_exposure() */
 
 static void
-frames_sort_test(const char *aDir, cpl_recipe *aRecipe)
+muse_test_processing_frames_sort_test(const char *aDir, cpl_recipe *aRecipe)
 {
 #ifdef HAVE_GETTIMEOFDAY /* fail/skip, if we don't have that function, see below */
   /*
@@ -220,9 +248,9 @@ frames_sort_test(const char *aDir, cpl_recipe *aRecipe)
   proc->inframes = cpl_frameset_duplicate(fall);
   table = muse_processing_sort_exposures(proc);
   cpl_test(cpl_table_get_nrow(table) == 3);
-  cpl_test(frames_sort_valid_ifus_in_exposure(table, 1) == 24);
-  cpl_test(frames_sort_valid_ifus_in_exposure(table, 2) == 24);
-  cpl_test(frames_sort_valid_ifus_in_exposure(table, 3) == 24);
+  cpl_test(muse_test_processing_frames_sort_valid_ifus_in_exposure(table, 1) == 24);
+  cpl_test(muse_test_processing_frames_sort_valid_ifus_in_exposure(table, 2) == 24);
+  cpl_test(muse_test_processing_frames_sort_valid_ifus_in_exposure(table, 3) == 24);
   cpl_table_delete(table);
 
   cpl_frameset_delete(proc->inframes);
@@ -230,18 +258,18 @@ frames_sort_test(const char *aDir, cpl_recipe *aRecipe)
   table = muse_processing_sort_exposures(proc);
   cpl_test(cpl_table_get_nrow(table) == 3);
   /* 2 IFUs should be missing for all exposures */
-  cpl_test(frames_sort_valid_ifus_in_exposure(table, 1) == 22);
-  cpl_test(frames_sort_valid_ifus_in_exposure(table, 2) == 22);
-  cpl_test(frames_sort_valid_ifus_in_exposure(table, 3) == 22);
+  cpl_test(muse_test_processing_frames_sort_valid_ifus_in_exposure(table, 1) == 22);
+  cpl_test(muse_test_processing_frames_sort_valid_ifus_in_exposure(table, 2) == 22);
+  cpl_test(muse_test_processing_frames_sort_valid_ifus_in_exposure(table, 3) == 22);
   cpl_table_delete(table);
 
   cpl_frameset_delete(proc->inframes);
   proc->inframes = cpl_frameset_duplicate(f2);
   table = muse_processing_sort_exposures(proc);
   cpl_test(cpl_table_get_nrow(table) == 3);
-  cpl_test(frames_sort_valid_ifus_in_exposure(table, 1) == 24);
-  cpl_test(frames_sort_valid_ifus_in_exposure(table, 2) == 12); /* every second one missing */
-  cpl_test(frames_sort_valid_ifus_in_exposure(table, 3) == 8); /* only every third is there */
+  cpl_test(muse_test_processing_frames_sort_valid_ifus_in_exposure(table, 1) == 24);
+  cpl_test(muse_test_processing_frames_sort_valid_ifus_in_exposure(table, 2) == 12); /* every second one missing */
+  cpl_test(muse_test_processing_frames_sort_valid_ifus_in_exposure(table, 3) == 8); /* only every third is there */
   cpl_table_delete(table);
 
   /* remove all files from the disk, using the frameset that contains all frames */
@@ -264,7 +292,7 @@ frames_sort_test(const char *aDir, cpl_recipe *aRecipe)
 #else /* no gettimeofday() available */
   cpl_msg_error(__func__, "gettimeofday() not available, cannot run this test!");
 #endif
-} /* frames_sort_test() */
+} /* muse_test_processing_frames_sort_test() */
 
 /*----------------------------------------------------------------------------*/
 /**
@@ -280,7 +308,7 @@ int main(int argc, char **argv)
   char *dir = mkdtemp(dirtemplate);
   int retval = chdir(dir);
   // test preparation: build input and test images
-  prepare_test(dir);
+  muse_test_processing_prepare_test(dir);
 
   // here we just simulate a framework.
   // initialize plugin list
@@ -298,31 +326,96 @@ int main(int argc, char **argv)
   retval = init_func(plugin);
   cpl_test_eq(retval, CPL_ERROR_NONE);
 
+  // test muse_processing_recipeinfo(), when no frames are there yet
+  cpl_errorstate state = cpl_errorstate_get();
+  muse_processing_recipeinfo(plugin);
+  cpl_test(cpl_errorstate_is_equal(state));
+
   // init (input) frames
   recipe->frames = cpl_frameset_new();
-
-  char *file = cpl_sprintf("%s/raw.fits", dir);
+  // create a RAW frame
+  char *file1 = cpl_sprintf("%s/raw.fits", dir);
   cpl_frame *frame = cpl_frame_new();
   cpl_frame_set_tag(frame, "RAW");
   cpl_frame_set_group(frame, CPL_FRAME_GROUP_RAW);
-  cpl_frame_set_filename(frame, file);
+  cpl_frame_set_filename(frame, file1);
   cpl_frameset_insert(recipe->frames, frame);
 
-  // run plugin
+  // and two tables
+  char *file2 = cpl_sprintf("%s/table.fits", dir);
+  frame = cpl_frame_new();
+  cpl_frame_set_tag(frame, "TABLE");
+  cpl_frame_set_group(frame, CPL_FRAME_GROUP_CALIB);
+  cpl_frame_set_filename(frame, file2);
+  cpl_frameset_insert(recipe->frames, frame);
+  char *file3 = cpl_sprintf("%s/tableifu.fits", dir);
+  frame = cpl_frame_new();
+  cpl_frame_set_tag(frame, "TABLE_IFU");
+  cpl_frame_set_group(frame, CPL_FRAME_GROUP_CALIB);
+  cpl_frame_set_filename(frame, file3);
+  cpl_frameset_insert(recipe->frames, frame);
+
+  // and a bare FITS header
+  char *file4 = cpl_sprintf("%s/header.fits", dir);
+  frame = cpl_frame_new();
+  cpl_frame_set_tag(frame, "HEADER");
+  cpl_frame_set_group(frame, CPL_FRAME_GROUP_CALIB);
+  cpl_frame_set_filename(frame, file4);
+  cpl_frameset_insert(recipe->frames, frame);
+
+#if 0
+  cpl_frameset_dump(recipe->frames, stdout);
+  fflush(stdout);
+  char *cmd = cpl_sprintf("ls -la %s", dir);
+  system(cmd);
+  cpl_free(cmd);
+#endif
+
+  // test muse_processing_recipeinfo(), now that the frames are there
+  state = cpl_errorstate_get();
+  muse_processing_recipeinfo(plugin);
+  cpl_test(cpl_errorstate_is_equal(state));
+  // again with some environment variables set and a non-default parameter
+  cpl_test_zero(setenv("OMP_NUM_THREADS", "5", 1));
+  cpl_test_zero(setenv("OMP_DYNNAMIC", "TRUE", 1)); // with typo
+  cpl_test_zero(setenv("MUSE_CPL_ERRORSTATE_NDUMP", "500", 1));
+  cpl_test_zero(setenv("MUSE_DAR_CORRECT_METHOD", "Ciddor", 1));
+  cpl_test_zero(setenv("MUSE_EXPERT", "1", 1)); // non-existant!
+  cpl_parameter *p = cpl_parameterlist_find(recipe->parameters,
+                                            "muse.muse_test_processing.astring");
+  char *pstring = cpl_strdup(cpl_parameter_get_string(p));
+  cpl_parameter_set_string(p, "BLA");
+  state = cpl_errorstate_get();
+  muse_processing_recipeinfo(plugin);
+  cpl_test(cpl_errorstate_is_equal(state));
+  // check visually, that the log does not print the ones with typos
+  cpl_test_zero(unsetenv("OMP_NUM_THREADS"));
+  cpl_test_zero(unsetenv("OMP_DYNNAMIC"));
+  cpl_test_zero(unsetenv("MUSE_CPL_ERRORSTATE_NDUMP"));
+  cpl_test_zero(unsetenv("MUSE_DAR_CORRECT_METHOD"));
+  cpl_test_zero(unsetenv("MUSE_EXPERT"));
+  cpl_parameter_set_string(p, pstring);
+  cpl_free(pstring);
+  // case for early-return
+  state = cpl_errorstate_get();
+  muse_processing_recipeinfo(NULL);
+  cpl_test(cpl_errorstate_is_equal(state));
+
+  // run plugin once
   cpl_plugin_func exec_func = cpl_plugin_get_exec(plugin);
   cpl_test(exec_func != NULL);
   retval = exec_func(plugin);
   cpl_test_eq(retval, CPL_ERROR_NONE);
 
-  // run plugin with debug-level output
+  // run plugin again, with debug-level output
   cpl_msg_set_level(CPL_MSG_DEBUG);
   retval = exec_func(plugin);
   cpl_test_eq(retval, CPL_ERROR_NONE);
   cpl_msg_set_level(CPL_MSG_INFO);
 
   // before removing it, use the still existing recipe to do
-  // the frames_sort_test that needs a new processing structure
-  frames_sort_test(dir, recipe);
+  // the frames-sort test that needs a new processing structure
+  muse_test_processing_frames_sort_test(dir, recipe);
 
   // cleanup
   cpl_plugin_func deinit_func = cpl_plugin_get_deinit(plugin);
@@ -333,8 +426,14 @@ int main(int argc, char **argv)
 
   cpl_pluginlist_delete(list);
   muse_processinginfo_delete(recipe);
-  remove(file);
-  cpl_free(file);
+  cpl_test_zero(remove(file1));
+  cpl_free(file1);
+  cpl_test_zero(remove(file2));
+  cpl_free(file2);
+  cpl_test_zero(remove(file3));
+  cpl_free(file3);
+  cpl_test_zero(remove(file4));
+  cpl_free(file4);
 
   // this removes the dir created for the results, and it also (implicitely)
   // checks that there are no more files.
@@ -363,6 +462,7 @@ muse_test_processing_compute(muse_processing *aProcessing,
   cpl_test_eq_string(aParams->astring, "DEF");
   cpl_test_eq(aParams->anenum, 1);
 
+  // check loading of raw data
   muse_imagelist *images = muse_basicproc_load(aProcessing, aParams->nifu,
                                                NULL);
   cpl_test_nonnull(images);
@@ -378,6 +478,49 @@ muse_test_processing_compute(muse_processing *aProcessing,
   cpl_propertylist_append_string(image->header, "BUNIT", "adu");
   int retval;
 
+  // loading of non-existing optional input is implicitly
+  // checked during recipe initialization
+
+  // check loading of tables
+  cpl_table *table = muse_processing_load_ctable(aProcessing, "TABLE", 0);
+  cpl_test_error(CPL_ERROR_NONE);
+  cpl_test_nonnull(table);
+  cpl_table_delete(table);
+  table = muse_processing_load_ctable(aProcessing, "TABLE_IFU", 1); /* IFU-specific */
+  cpl_test_error(CPL_ERROR_NONE);
+  cpl_test_nonnull(table);
+  cpl_table_delete(table);
+  // failure case for generic file
+  // (this and the following failure cases implicitly also
+  // test muse_processing_load_table())
+  cpl_table *table2 = muse_processing_load_ctable(aProcessing,
+                                                  "TABLE_NOT_PRESENT", 0);
+  cpl_test_error(CPL_ERROR_NONE); /* still no error, since this may be optional */
+  cpl_test_null(table2);
+  // failure case for table with IFU
+  table2 = muse_processing_load_ctable(aProcessing, "TABLE_NOT_PRESENT", 10);
+  cpl_test_error(CPL_ERROR_NONE); /* no error, since this may be optional */
+  cpl_test_null(table2);
+  // failure case of non-existant file
+  // to do that, add another entry to the inframes and remove it again afterwards
+  cpl_frameset *fin = cpl_frameset_duplicate(aProcessing->inframes); // save them
+  cpl_frame *ffake = cpl_frame_new();
+  cpl_frame_set_filename(ffake, "non-exiting.fits");
+  cpl_frame_set_tag(ffake, "TABLE_FAKE_PRESENT");
+  cpl_frameset_insert(aProcessing->inframes, ffake);
+  table2 = muse_processing_load_ctable(aProcessing, "TABLE_FAKE_PRESENT", 0);
+  cpl_test_error(CPL_ERROR_FILE_IO);
+  cpl_test_null(table2);
+  cpl_frameset_delete(aProcessing->inframes);
+  aProcessing->inframes = fin;
+  fin = NULL;
+
+  // check loading of propertylists (like ASTROMETRY_WCS)
+  cpl_propertylist *header = muse_processing_load_header(aProcessing, "HEADER");
+  cpl_test_error(CPL_ERROR_NONE);
+  cpl_test_nonnull(header);
+  cpl_propertylist_delete(header);
+
   // check saving of MUSE images
   retval = muse_processing_save_image(aProcessing, aParams->nifu,
                                       image, "processing_muse_image");
@@ -390,7 +533,7 @@ muse_test_processing_compute(muse_processing *aProcessing,
   cpl_test_error(CPL_ERROR_NONE);
 
   // check saving of CPL tables
-  cpl_table *table = cpl_table_new(0);
+  table = cpl_table_new(0); // will trigger case "has no rows"
   retval = muse_processing_save_table(aProcessing, aParams->nifu,
                                       table, image->header,
                                       "processing_cpl_table",
@@ -398,6 +541,25 @@ muse_test_processing_compute(muse_processing *aProcessing,
   cpl_test_error(CPL_ERROR_NONE);
   cpl_table_delete(table);
 
+  // check saving of MUSE tables
+  muse_table *mtable = muse_table_new();
+  mtable->table = cpl_table_new(1);
+  mtable->header = cpl_propertylist_new(); // do not set AFLOAT, to trigger warning
+  cpl_propertylist_append_string(mtable->header, "OBJECT", "A test MUSE table");
+  cpl_propertylist_append_string(mtable->header, "ATABLE", "A stipud string...");
+  retval = muse_processing_save_table(aProcessing, 0, mtable, NULL,
+                                      "processing_muse_table",
+                                      MUSE_TABLE_TYPE_MUSE);
+  cpl_test_error(CPL_ERROR_NONE);
+  // test failure case
+  cpl_table_delete(mtable->table);
+  mtable->table = NULL;
+  retval = muse_processing_save_table(aProcessing, 0, mtable, NULL,
+                                      "processing_muse_table_BAD",
+                                      MUSE_TABLE_TYPE_MUSE);
+  cpl_test_error(CPL_ERROR_NULL_INPUT);
+  muse_table_delete(mtable);
+
   // check saving of FITS headers
   retval = muse_processing_save_header(aProcessing, aParams->nifu,
                                        image->header, "processing_header");
@@ -433,6 +595,13 @@ muse_test_processing_compute(muse_processing *aProcessing,
                                       "processing_data_cube",
                                       MUSE_CUBE_TYPE_FITS);
   cpl_test_error(CPL_ERROR_NONE);
+  // try saving a cube with IDP headers (supposed to fail!)
+  cpl_propertylist_append_string(cube->header, "DATE-OBS",
+                                 "2016-09-14T12:30:00.002");
+  retval = muse_processing_save_cube(aProcessing, -1, cube,
+                                      "processing_data_cube_with_idp_BAD",
+                                      MUSE_CUBE_TYPE_SDP);
+  cpl_test_error(CPL_ERROR_DATA_NOT_FOUND); /* missing important headers */
   muse_datacube_delete(cube);
 
   // check saving of MUSE masks
@@ -440,26 +609,31 @@ muse_test_processing_compute(muse_processing *aProcessing,
   mask->header = cpl_propertylist_new();
   cpl_propertylist_append_string(mask->header, "OBJECT", "A MASK");
   mask->mask = cpl_mask_new(10, 10);
+  cpl_mask_set(mask->mask, 3, 3, CPL_BINARY_1);
   retval = muse_processing_save_mask(aProcessing, -1, mask,
                                      "processing_mask");
   cpl_test_error(CPL_ERROR_NONE);
   muse_mask_delete(mask);
   mask = NULL;
 
-#if 0
-  // check loading the MUSE mask back
-  // XXX this doesn't work with the current XML config for this test and
-  //     it would need pre-existing data that somehow gets set in the input SOF
-  mask = muse_processing_mask_load(aProcessing, "processing_mask");
-  cpl_test_error(CPL_ERROR_NONE);
-  cpl_test_nonnull(mask);
-  cpl_test_nonnull(mask->header);
-  cpl_test_nonnull(mask->mask);
-  muse_mask_delete(mask);
-#endif
+  // check loading the MUSE mask back (this only works on the 2nd run!)
+  cpl_errorstate state = cpl_errorstate_get();
+  mask = muse_processing_load_mask(aProcessing, "processing_mask");
+  if (mask) {
+    cpl_test_error(CPL_ERROR_NONE);
+    cpl_test_nonnull(mask);
+    cpl_test_nonnull(mask->header);
+    cpl_test_nonnull(mask->mask);
+    muse_mask_delete(mask);
+    /* in this case, remove the loaded frame again from the used frames, *
+     * so that the CPL DFS functions tested below work the same way      */
+    cpl_frameset_erase(aProcessing->usedframes, "processing_mask");
+  } else {
+    cpl_errorstate_set(state);
+  }
 
   // check the output frameset
-  cpl_test_eq(cpl_frameset_get_size(aProcessing->outframes), 7);
+  cpl_test_eq(cpl_frameset_get_size(aProcessing->outframes), 8);
   cpl_frame *frame;
   frame = cpl_frameset_find(aProcessing->outframes, "processing_muse_image");
   cpl_test_nonnull(frame);
@@ -544,7 +718,7 @@ muse_test_processing_compute(muse_processing *aProcessing,
 
   /* Test some failure cases. All these functions just output   *
    * ERROR messages, they don't set error states by themselves. */
-  cpl_errorstate state = cpl_errorstate_get();
+  state = cpl_errorstate_get();
   muse_processing_delete(NULL);
   cpl_test(cpl_errorstate_is_equal(state));
   /* try to create a frame, but first fail without processing struct */
@@ -564,8 +738,9 @@ muse_test_processing_compute(muse_processing *aProcessing,
   cpl_test_nonnull(testframe); /* this time it should have succeeded */
   cpl_test(!cpl_propertylist_has(hdr, "MUSE PRIVATE SOMETHING")); /* be removed */
   /* now use this frame to generate a failure when appending used */
+  state = cpl_errorstate_get();
   muse_processing_append_used(NULL, testframe, CPL_FRAME_GROUP_RAW, 1);
-  cpl_test(cpl_errorstate_is_equal(state));
+  cpl_test(cpl_errorstate_is_equal(state)); /* just silently fails */
   cpl_frame_delete(testframe);
   cpl_propertylist_delete(hdr);
   /* one last check, of muse_processing_check_input() */
