@@ -682,53 +682,6 @@ muse_test_resampling_cube(muse_pixtable *aPtPix)
           cpl_errorstate_set(state);
         } /* if drizzle */
 
-        /* for one of the tests, also create a mask cube and test the bad pixel table creation this way */
-        if (t == MUSE_RESAMPLE_WEIGHTED_DRIZZLE &&
-            tl == MUSE_RESAMPLING_DISP_AWAV && ipt == 0) {
-          setenv("MUSE_BADPIX_FROM_MASKCUBE",
-                 "maskcube_test.fits,badpix_test.fits", 1);
-          /* create the mask cube */
-          int nx = cpl_image_get_size_x(cpl_imagelist_get(cube->data, 0)),
-              ny = cpl_image_get_size_y(cpl_imagelist_get(cube->data, 0));
-          cpl_image *mask = cpl_image_new(nx, ny, CPL_TYPE_INT);
-          cpl_image_fill_window(mask, 8, 3, 12, 5, 1);
-          cpl_imagelist *maskcube = cpl_imagelist_new();
-          int i, err;
-          for (i = 0; i < nplanes; i++) {
-            cpl_imagelist_set(maskcube, mask, i);
-          } /* for i (all planes) */
-          /* different mask for the next-to-last plane */
-          cpl_image *mask2 = cpl_image_new(nx, ny, CPL_TYPE_INT);
-          cpl_image_fill_window(mask2, 10, 5, 30, 9, 1);
-          cpl_imagelist_set(maskcube, mask2, nplanes - 2);
-          /* and save the masking cube */
-          cpl_imagelist_save(maskcube, "maskcube_test.fits", CPL_TYPE_INT, NULL,
-                             CPL_IO_CREATE);
-          cpl_imagelist_delete(maskcube);
-          cpl_test_zero(access("maskcube_test.fits", F_OK));
-          /* now do the resampling with bad pixel mask creation */
-          muse_resampling_params *parnocr = muse_resampling_params_new(t);
-          parnocr->tlambda = tl;
-          parnocr->crsigma = -1.;
-          muse_datacube *cb = muse_resampling_cube(pt, parnocr, NULL);
-          muse_resampling_params_delete(parnocr);
-          cpl_test(isnan(cpl_image_get(cpl_imagelist_get(cb->stat, 3), 8, 3, &err)));
-          cpl_test(isnan(cpl_image_get(cpl_imagelist_get(cb->stat, 3), 12, 5, &err)));
-          cpl_test(!isnan(cpl_image_get(cpl_imagelist_get(cb->stat, 3), 7, 2, &err)));
-          cpl_test(!isnan(cpl_image_get(cpl_imagelist_get(cb->stat, 3), 13, 6, &err)));
-          cpl_test(isnan(cpl_image_get(cpl_imagelist_get(cb->stat, nplanes - 2), 10, 5, &err)));
-          cpl_test(isnan(cpl_image_get(cpl_imagelist_get(cb->stat, nplanes - 2), 30, 9, &err)));
-          muse_datacube_save(cb, "test_cube.fits");
-          muse_datacube_delete(cb);
-          cpl_test_zero(access("badpix_test.fits", F_OK));
-#if MUSE_TEST_RESAMPLING_DELETE_FILES
-          cpl_test(remove("maskcube_test.fits") == 0);
-          cpl_test(remove("test_cube.fits") == 0);
-          cpl_test(remove("badpix_test.fits") == 0);
-#endif
-          cpl_test(unsetenv("MUSE_BADPIX_FROM_MASKCUBE") == 0);
-        }
-
         /* now we are done with both cubes and the parameters */
         prestate = cpl_errorstate_get();
         muse_datacube_delete(cube);
@@ -837,6 +790,43 @@ muse_test_resampling_cube(muse_pixtable *aPtPix)
   cpl_test_nonnull(params->wcs);
   cpl_test(params->tlambda == MUSE_RESAMPLING_DISP_AWAV_LOG);
   cpl_propertylist_delete(header);
+  /* test setting pixfrac */
+  cpl_test_rel(params->pfx, 0.6, DBL_EPSILON);
+  cpl_test_rel(params->pfy, 0.6, DBL_EPSILON);
+  cpl_test_rel(params->pfl, 0.6, DBL_EPSILON);
+  cpl_test(muse_resampling_params_set_pixfrac(params, "0.8") == CPL_ERROR_NONE);
+  cpl_test_rel(params->pfx, 0.8, DBL_EPSILON);
+  cpl_test_rel(params->pfy, 0.8, DBL_EPSILON);
+  cpl_test_rel(params->pfl, 0.8, DBL_EPSILON);
+  cpl_test(muse_resampling_params_set_pixfrac(params, "0.6,0.8")
+           == CPL_ERROR_NONE);
+  cpl_test_rel(params->pfx, 0.6, DBL_EPSILON);
+  cpl_test_rel(params->pfy, 0.6, DBL_EPSILON);
+  cpl_test_rel(params->pfl, 0.8, DBL_EPSILON);
+  cpl_test(muse_resampling_params_set_pixfrac(params, "0.6,0.65,0.8")
+           == CPL_ERROR_NONE);
+  cpl_test_rel(params->pfx, 0.6, DBL_EPSILON);
+  cpl_test_rel(params->pfy, 0.65, DBL_EPSILON);
+  cpl_test_rel(params->pfl, 0.8, DBL_EPSILON);
+  ps = cpl_errorstate_get();
+  cpl_test(muse_resampling_params_set_pixfrac(params, NULL)
+           == CPL_ERROR_NULL_INPUT);
+  cpl_errorstate_set(ps);
+  cpl_test(muse_resampling_params_set_pixfrac(NULL, "0.8")
+           == CPL_ERROR_NULL_INPUT);
+  cpl_errorstate_set(ps);
+  cpl_test(muse_resampling_params_set_pixfrac(params, "")
+           == CPL_ERROR_ILLEGAL_INPUT);
+  cpl_errorstate_set(ps);
+  cpl_test_rel(params->pfx, 0.6, DBL_EPSILON);
+  cpl_test_rel(params->pfy, 0.65, DBL_EPSILON);
+  cpl_test_rel(params->pfl, 0.8, DBL_EPSILON);
+  cpl_test(muse_resampling_params_set_pixfrac(params, "0.1,0.2,0.3,0.4")
+           == CPL_ERROR_ILLEGAL_INPUT);
+  cpl_errorstate_set(ps);
+  cpl_test_rel(params->pfx, 0.6, DBL_EPSILON);
+  cpl_test_rel(params->pfy, 0.65, DBL_EPSILON);
+  cpl_test_rel(params->pfl, 0.8, DBL_EPSILON);
   muse_resampling_params_delete(params);
   /* parameter deletion is also supposed to work with NULL argument */
   ps = cpl_errorstate_get();
