@@ -2,7 +2,7 @@
 /* vim:set sw=2 sts=2 et cin: */
 /*
  * This file is part of the MUSE Instrument Pipeline
- * Copyright (C) 2007-2015 European Southern Observatory
+ * Copyright (C) 2007-2017 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -58,9 +58,10 @@ int main(int argc, char **argv)
   } else {
     arc = muse_image_load(BASEFILENAME"_in.fits");
   }
-  /* make sure that the IFU number can be determined */
+  /* make sure that the IFU number can be determined and a mode */
   if (!cpl_propertylist_has(arc->header, "EXTNAME")) {
     cpl_propertylist_update_string(arc->header, "EXTNAME", "CHAN10");
+    cpl_propertylist_update_string(arc->header, "ESO INS MODE", "WFM-NOAO-N");
   }
   cpl_error_reset();
   cpl_table *tracetable = cpl_table_load(BASEFILENAME"_trace_in.fits", 1, 1);
@@ -76,26 +77,35 @@ int main(int argc, char **argv)
     return cpl_test_end(2);
   }
 
-  /* use default values from muse_wavecal */
-  muse_wave_params *p = muse_wave_params_new();
-  p->rflag = CPL_TRUE; /* want residuals table */
-  /* we loop over the weighting type in the muse_test_wavecal_arc_lampwise *
-   * program, so here just use the non-weighted part for quick runtime     */
-  p->fitweighting = MUSE_WAVE_WEIGHTING_UNIFORM;
-  double cputime1 = cpl_test_get_cputime(),
-         time1 = cpl_test_get_walltime();
+  /* some variables upfront */
   cpl_errorstate state = cpl_errorstate_get();
-  cpl_table *wavecaltable = muse_wave_calib(arc, tracetable, linelist->table,
-                                            p);
-  double cputime2 = cpl_test_get_cputime(),
-         time2 = cpl_test_get_walltime();
-  cpl_msg_debug(__func__, "finished muse_wave_calib(), took %gs (CPU) %gs "
-                "(wall-clock)", cputime2 - cputime1, time2 - time1);
-  cpl_test(cpl_errorstate_is_equal(state));
-  cpl_test_nonnull(wavecaltable);
-  cpl_test_nonnull(p->residuals);
-  if (wavecaltable) {
-    cpl_msg_info(__func__, "wavecaltable created");
+  cpl_table *wavecaltable = NULL;
+  /* test with all instrument modes */
+  int imode,
+      nlines[] = { 50, 49, 44, 45, 45 }; /* number of detected lines per mode */
+  for (imode = 0; imode <= MUSE_MODE_NFM_AO_N; imode++) {
+    /* use default values from muse_wavecal */
+    muse_wave_params *p = muse_wave_params_new(NULL); /* test without header */
+    p->rflag = CPL_TRUE; /* want residuals table */
+    p->mode = imode; /* pretend that the test data is of some other mode */
+    /* we loop over the weighting type in the muse_test_wavecal_arc_lampwise *
+     * program, so here just use the non-weighted part for quick runtime     */
+    p->fitweighting = MUSE_WAVE_WEIGHTING_UNIFORM;
+    double cputime1 = cpl_test_get_cputime(),
+           time1 = cpl_test_get_walltime();
+    state = cpl_errorstate_get();
+    wavecaltable = muse_wave_calib(arc, tracetable, linelist->table, p);
+    double cputime2 = cpl_test_get_cputime(),
+           time2 = cpl_test_get_walltime();
+    cpl_msg_debug(__func__, "finished muse_wave_calib(), took %gs (CPU) %gs "
+                  "(wall-clock)", cputime2 - cputime1, time2 - time1);
+    cpl_test(cpl_errorstate_is_equal(state));
+    cpl_test_nonnull(wavecaltable);
+    cpl_test_nonnull(p->residuals);
+    if (!wavecaltable) {
+      continue;
+    }
+    cpl_msg_info(__func__, "mode %d wavecaltable created", imode);
     cpl_test(cpl_table_get_nrow(wavecaltable) == kMuseSlicesPerCCD);
     /* there should be no invalid elements in the low-order coefficients *
      * nor in the mean squared error column                              */
@@ -120,6 +130,14 @@ int main(int argc, char **argv)
       cpl_test(wlen > 6810. && wlen < 7190.);
       cpl_test(rms < 0.125); /* loose check, following PR27 */
     }
+    char *keyword = cpl_sprintf(QC_WAVECAL_SLICEj_FIT_NLINES, 10);
+    int nfit = cpl_propertylist_get_int(arc->header, keyword);
+    cpl_free(keyword);
+#if 0
+    cpl_msg_debug(__func__, "Detected %d lines in slice 10 (should be %d)",
+                  nfit, nlines[imode]);
+#endif
+    cpl_test_eq(nfit, nlines[imode]); /* that's what I see after a first test */
 
     cpl_test(cpl_table_get_nrow(p->residuals) > 0);
     double meanlambda = cpl_table_get_column_mean(p->residuals, "lambda");
@@ -127,43 +145,47 @@ int main(int argc, char **argv)
     double meanresiduals = cpl_table_get_column_mean(p->residuals, "residual");
     /* mean of all residuals should be much smaller than 0.1 Angstrom */
     cpl_test(meanresiduals > -0.1 && meanresiduals < 0.1);
+    muse_wave_params_delete(p);
 
-    /* check muse_wave_table_get_poly_for_slice() *
-     * start with failure cases                   */
-    state = cpl_errorstate_get();
-    cpl_polynomial *pwave = muse_wave_table_get_poly_for_slice(NULL, 1);
-    cpl_test_null(pwave);
-    cpl_test(!cpl_errorstate_is_equal(state) &&
-             cpl_error_get_code() == CPL_ERROR_NULL_INPUT);
-    cpl_errorstate_set(state);
-    pwave = muse_wave_table_get_poly_for_slice(wavecaltable, 0);
-    cpl_test_null(pwave);
-    cpl_test(!cpl_errorstate_is_equal(state) &&
-             cpl_error_get_code() == CPL_ERROR_ILLEGAL_INPUT);
-    cpl_errorstate_set(state);
-    pwave = muse_wave_table_get_poly_for_slice(wavecaltable,
-                                               kMuseSlicesPerCCD + 1);
-    cpl_test_null(pwave);
-    cpl_test(!cpl_errorstate_is_equal(state) &&
-             cpl_error_get_code() == CPL_ERROR_ILLEGAL_INPUT);
-    cpl_errorstate_set(state);
-    cpl_table *wvbroken = cpl_table_duplicate(wavecaltable);
-    /* delete last three slices: */
-    cpl_table_set_size(wvbroken, kMuseSlicesPerCCD - 3);
-    /* test next to last slice; */
-    pwave = muse_wave_table_get_poly_for_slice(wvbroken, kMuseSlicesPerCCD - 1);
-    cpl_test_null(pwave);
-    cpl_test(!cpl_errorstate_is_equal(state) &&
-             cpl_error_get_code() == CPL_ERROR_DATA_NOT_FOUND);
-    cpl_errorstate_set(state);
-    /* break double quadratic coeff in slice 1 */
-    cpl_table_set_invalid(wvbroken, "wlc22", 0);
-    pwave = muse_wave_table_get_poly_for_slice(wvbroken, 1);
-    cpl_test_null(pwave);
-    cpl_test(!cpl_errorstate_is_equal(state) &&
-             cpl_error_get_code() == CPL_ERROR_ILLEGAL_OUTPUT);
-    cpl_errorstate_set(state);
-    cpl_table_delete(wvbroken);
+    /* check muse_wave_table_get_poly_for_slice() */
+    cpl_polynomial *pwave = NULL;
+    if (imode == 0) {
+      /* start with failure cases                   */
+      state = cpl_errorstate_get();
+      pwave = muse_wave_table_get_poly_for_slice(NULL, 1);
+      cpl_test_null(pwave);
+      cpl_test(!cpl_errorstate_is_equal(state) &&
+               cpl_error_get_code() == CPL_ERROR_NULL_INPUT);
+      cpl_errorstate_set(state);
+      pwave = muse_wave_table_get_poly_for_slice(wavecaltable, 0);
+      cpl_test_null(pwave);
+      cpl_test(!cpl_errorstate_is_equal(state) &&
+               cpl_error_get_code() == CPL_ERROR_ILLEGAL_INPUT);
+      cpl_errorstate_set(state);
+      pwave = muse_wave_table_get_poly_for_slice(wavecaltable,
+                                                 kMuseSlicesPerCCD + 1);
+      cpl_test_null(pwave);
+      cpl_test(!cpl_errorstate_is_equal(state) &&
+               cpl_error_get_code() == CPL_ERROR_ILLEGAL_INPUT);
+      cpl_errorstate_set(state);
+      cpl_table *wvbroken = cpl_table_duplicate(wavecaltable);
+      /* delete last three slices: */
+      cpl_table_set_size(wvbroken, kMuseSlicesPerCCD - 3);
+      /* test next to last slice; */
+      pwave = muse_wave_table_get_poly_for_slice(wvbroken, kMuseSlicesPerCCD - 1);
+      cpl_test_null(pwave);
+      cpl_test(!cpl_errorstate_is_equal(state) &&
+               cpl_error_get_code() == CPL_ERROR_DATA_NOT_FOUND);
+      cpl_errorstate_set(state);
+      /* break double quadratic coeff in slice 1 */
+      cpl_table_set_invalid(wvbroken, "wlc22", 0);
+      pwave = muse_wave_table_get_poly_for_slice(wvbroken, 1);
+      cpl_test_null(pwave);
+      cpl_test(!cpl_errorstate_is_equal(state) &&
+               cpl_error_get_code() == CPL_ERROR_ILLEGAL_OUTPUT);
+      cpl_errorstate_set(state);
+      cpl_table_delete(wvbroken);
+    } /* imode==0 (failure cases) */
     /* check successes */
     pwave = muse_wave_table_get_poly_for_slice(wavecaltable, kMuseSlicesPerCCD);
     cpl_test_nonnull(pwave);
@@ -186,9 +208,11 @@ int main(int argc, char **argv)
     cpl_polynomial_delete(pwave);
 
     cpl_table_delete(wavecaltable);
-  } /* if wavecaltable */
+  } /* for imode (all instrument modes) */
+
   /* test failure cases */
   state = cpl_errorstate_get();
+  muse_wave_params *p = muse_wave_params_new(NULL);
   wavecaltable = muse_wave_calib(NULL, tracetable, linelist->table, p);
   cpl_test(!cpl_errorstate_is_equal(state)
            && cpl_error_get_code() == CPL_ERROR_NULL_INPUT);
