@@ -456,7 +456,7 @@ muse_test_wcs_calibrate_from_cube(int argc, char **argv)
   cpl_table_delete(reftable);
   muse_datacube_delete(cube);
 
-  return cpl_errorstate_is_equal(state) ? 0 : 50;
+  return cpl_errorstate_is_equal(state) ? CPL_ERROR_NONE : cpl_error_get_code();
 } /* muse_test_wcs_calibrate_from_cube() */
 
 /*----------------------------------------------------------------------------*/
@@ -492,14 +492,18 @@ muse_test_wcs_calibrate_from_cube(int argc, char **argv)
 /*----------------------------------------------------------------------------*/
 int main(int argc, char **argv)
 {
-  cpl_test_init(PACKAGE_BUGREPORT, CPL_MSG_DEBUG);
-
+  cpl_error_code rc = CPL_ERROR_NONE;
   if (argc > 1) {
     /* If there are arguments, misuse this program as astrometry calibrator. */
-    cpl_test(muse_test_wcs_calibrate_from_cube(argc, argv) == 0);
-    return cpl_test_end(0);
+    cpl_init(CPL_INIT_DEFAULT);
+    rc = muse_test_wcs_calibrate_from_cube(argc, argv);
+    cpl_end();
+    return rc;
   }
 
+  /* start as test program */
+  cpl_test_init(PACKAGE_BUGREPORT, CPL_MSG_DEBUG);
+
   /*********************************************
    * check WCS creation from input pixel table *
    *********************************************/
@@ -509,8 +513,7 @@ int main(int argc, char **argv)
   /* run once with muse_wcs_locate_sources() to create cube and image */
   muse_wcs_object *wcsobj = muse_wcs_object_new();
   cpl_test_nonnull(wcsobj);
-  cpl_error_code rc = muse_wcs_locate_sources(ptin, 3., MUSE_WCS_CENTROID_MOFFAT,
-                                              wcsobj);
+  rc = muse_wcs_locate_sources(ptin, 3., MUSE_WCS_CENTROID_MOFFAT, wcsobj);
   cpl_test(rc == CPL_ERROR_NONE);
   cpl_test_nonnull(wcsobj->cube);
   cpl_test_nonnull(wcsobj->detected);
