@@ -34,6 +34,8 @@
  * correspond to the one given in that entry.
  *
  * <b>Command line arguments:</b>
+ *   - <tt>-s</tt> (optional, only when compiled with MUSE_EXPERT=1)\n
+ *     sign a MUSE static calibration
  *   - <tt><b>FILENAME</b></tt>\n
  *     the filename of the MUSE pipeline product
  *   - <tt><b>TAG</b></tt>\n
@@ -49,9 +51,15 @@
 
 /**@{*/
 
-#define PRINT_USAGE(rc)                                                        \
-  fprintf(stderr, "Usage: %s FILENAME TAG\n", argv[0]);                        \
-  cpl_end(); return (rc);
+#ifdef MUSE_EXPERT /* extra -s flag for the maintainer! */
+  #define PRINT_USAGE(rc)                                                      \
+    fprintf(stderr, "Usage: %s [ -s ] FILENAME TAG\n", argv[0]);               \
+    cpl_end(); return (rc);
+#else /* normal compilation, when not maintainer */
+  #define PRINT_USAGE(rc)                                                      \
+    fprintf(stderr, "Usage: %s FILENAME TAG\n", argv[0]);                      \
+     cpl_end(); return (rc);
+#endif
 
 int main(int argc, char **argv)
 {
@@ -66,8 +74,14 @@ int main(int argc, char **argv)
   /* argument processing */
   char *iname = NULL, /* filename */
        *tag = NULL; /* tag */
+  cpl_boolean statcal = CPL_FALSE;
   int i;
   for (i = 1; i < argc; i++) {
+#ifdef MUSE_EXPERT /* extra -s flag for the maintainer! */
+    if (strncmp(argv[i], "-s", 3) == 0) {
+      statcal = CPL_TRUE;
+    } else /* if follows below*/
+#endif
     if (strncmp(argv[i], "-", 1) == 0) { /* unallowed options */
       PRINT_USAGE(9);
     } else {
@@ -103,7 +117,11 @@ int main(int argc, char **argv)
   cpl_frameset_dump(fset, stdout);
   fflush(stdout);
 #endif
-  cpl_dfs_sign_products(fset, CPL_DFS_SIGNATURE_DATAMD5 | CPL_DFS_SIGNATURE_CHECKSUM);
+  unsigned int flags = CPL_DFS_SIGNATURE_DATAMD5;
+  if (!statcal) {
+    flags |= CPL_DFS_SIGNATURE_CHECKSUM;
+  }
+  cpl_dfs_sign_products(fset, flags);
 #if 0
   printf("fset signed!\n");
   cpl_frameset_dump(fset, stdout);
@@ -111,6 +129,47 @@ int main(int argc, char **argv)
 #endif
   cpl_frameset_delete(fset);
 
+#ifdef MUSE_EXPERT /* special compilation for the maintainer! */
+  #include <fitsio.h>
+  /* special handling for static calibrations, since these cannot contain *
+   * PIPEFILE keywords which the CPL function above adds unconditionally; *
+   * since there is no way in CPL to rewrite the header with the correct  *
+   * CHECKSUM keyword but without changing the DATE, do it with CFITSIO   *
+   * directly                                                             */
+  if (statcal) {
+    printf("Handling \"%s\" as STATIC calibration!\n", iname);
+    int status;
+    fitsfile *fptr;
+    fits_open_file(&fptr, iname, READWRITE, &status);
+    if (status) {
+      fits_report_error(stderr, status);
+    }
+    /* primary HDU, no need to call fits_movabs_hdu() */
+    fits_delete_key(fptr, "PIPEFILE", &status);
+    if (status) {
+      fits_report_error(stderr, status);
+    }
+    /* add/overwrite key ESO.OBS.NAME */
+    fits_update_key_str(fptr, "HIERARCH ESO OBS NAME", "STATIC",
+                        "Static calibration for the MUSE pipeline.", &status);
+    // really insert at position before PRO.CATG, i.e. using
+    //   fits_insert_record(unit,key_no,card, &status);
+    // but this needs the position of PRO.CATG first
+    if (status) {
+      fits_report_error(stderr, status);
+    }
+    /* now finally recompute the CHECKSUM */
+    fits_write_chksum(fptr, &status);
+    if (status) {
+      fits_report_error(stderr, status);
+    }
+    fits_close_file(fptr, &status);
+    if (status) {
+      fits_report_error(stderr, status);
+    }
+  } /* if static calibration */
+#endif /* end of maintainer-only section */
+
   int rc = 0;
   if (!cpl_errorstate_is_equal(state)) {
     cpl_errorstate_dump(state, CPL_FALSE, muse_cplerrorstate_dump_some);
